@@ -414,7 +414,7 @@ static int do_single_transmit(sc_card_t *card, sc_apdu_t *apdu)
 			if (card->wait_resend_apdu != 0)
 				msleep(card->wait_resend_apdu);
 			/* re-transmit the APDU with new Le length */
-			r = do_single_transmit(card, apdu);
+			r = card->reader->ops->transmit(card->reader, card->slot, apdu);
 			if (r != SC_SUCCESS) {
 				sc_error(ctx, "unable to transmit APDU");
 				return r;
@@ -446,7 +446,7 @@ static int do_single_transmit(sc_card_t *card, sc_apdu_t *apdu)
 			 * requested or until the card retuns 0x9000, 
 			 * whatever happens first.
 			 */
-			size_t le, buflen;
+			size_t le, minlen, buflen;
 			u8     *buf;
 
 			if (card->ops->get_response == NULL) {
@@ -467,6 +467,9 @@ static int do_single_transmit(sc_card_t *card, sc_apdu_t *apdu)
 
 			/* 0x6100 means at least 256 more bytes to read */
 			le = apdu->sw2 != 0 ? (size_t)apdu->sw2 : 256;
+			/* we try to read at least as much as bytes as 
+			 * promised in the response bytes */
+			minlen = le;
 
 			do {
 				u8 tbuf[256];
@@ -483,7 +486,16 @@ static int do_single_transmit(sc_card_t *card, sc_apdu_t *apdu)
 				memcpy(buf, tbuf, le);
 				buf    += le;
 				buflen -= le;
-			} while (r != 0);
+
+				minlen -= le;
+				if (r != 0) 
+					le = minlen = (size_t)r;
+				else
+					/* if the card has returned 0x9000 but
+					 * we still expect data ask for more 
+					 * until we have read enough bytes */
+					le = minlen;
+			} while (r != 0 || minlen != 0);
 			/* we've read all data, let's return 0x9000 */
 			apdu->resplen = buf - apdu->resp;
 			apdu->sw1 = 0x90;
