@@ -108,7 +108,7 @@ static int jcop_init(sc_card_t *card)
      drvdata->selected=SELECT_MF;
      drvdata->invalid_senv=1;
      drvdata->nfiles=-1;
-     drvdata->filelist=0;
+     drvdata->filelist=NULL;
      f=sc_file_new();
      if (!f){
 	  free(drvdata);
@@ -428,7 +428,7 @@ static int jcop_process_fci(sc_card_t *card, sc_file_t *file,
 	  if (drvdata->nfiles) {
 	       drvdata->nfiles=-1;
 	       free(drvdata->filelist);
-	       drvdata->filelist=0;
+	       drvdata->filelist=NULL;
 	  }
 	  if(file->sec_attr_len >=3) {
 	       /* The security attribute bytes are divided into nibbles and are
@@ -459,7 +459,7 @@ static int jcop_process_fci(sc_card_t *card, sc_file_t *file,
 	  if (drvdata->nfiles) {
 	       drvdata->nfiles=0;
 	       free(drvdata->filelist);
-	       drvdata->filelist=0;
+	       drvdata->filelist=NULL;
 	  }    
 	  /* the format of the poprietary attributes is:
 	     4 bytes     unique id
@@ -921,16 +921,11 @@ static int jcop_card_ctl(sc_card_t *card, unsigned long cmd, void *ptr)
         return SC_ERROR_NOT_SUPPORTED;
 }
 
-
 /* "The PINs are "global" in a PKCS#15 sense, meaning that they remain valid
  *  until card reset! Selecting another applet doesn't invalidate the PINs, 
  *  you need to reset the card." - javacard@zurich.ibm.com, when asked about 
  *  how to invalidate logged in pins.
-*/
-static int jcop_logout(sc_card_t *card)
-{
-     return 0; /* Can't */
-}
+ */
 
 static struct sc_card_driver * sc_get_driver(void)
 {
@@ -954,7 +949,6 @@ static struct sc_card_driver * sc_get_driver(void)
      jcop_ops.set_security_env = jcop_set_security_env;
      jcop_ops.compute_signature = jcop_compute_signature;
      jcop_ops.decipher = jcop_decipher;
-     jcop_ops.logout = jcop_logout;
      jcop_ops.process_fci = jcop_process_fci;
      jcop_ops.card_ctl = jcop_card_ctl;
      
