@@ -2,8 +2,10 @@
  * card-piv.c: Support for PIV-II from NIST SP800-73 
  * card-default.c: Support for cards with no driver
  *
- * Copyright (C) 2001, 2002  Juha Yrjl <juha.yrjola@iki.fi>
- * Copyright (C) 2005, Douglas E. Engert <deengert@anl.gov>
+ * Copyright (C) 2001, 2002  Juha Yrjölä <juha.yrjola@iki.fi>
+ * Copyright (C) 2005,2006,2007 Douglas E. Engert <deengert@anl.gov>
+ * Copyright (C) 2006, Identity Alliance, Thomas Harning <thomas.harning@identityalliance.com>
+ * Copyright (C) 2007, EMC, Russell Larner <rlarner@rsa.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,16 +23,26 @@
  */
 
 #include "internal.h"
+#include <ctype.h>
 #include <string.h>
 #include <fcntl.h>
-#include <openssl/des.h>
+#include <openssl/evp.h>
 #include <openssl/bio.h>
 #include <openssl/pem.h>
 #include <openssl/rsa.h>
 #include "asn1.h"
 #include "cardctl.h"
+#ifdef HAVE_ZLIB_H
+#include "compression.h"
+#endif
 
-struct piv_private_data {
+typedef struct {
+	u8* data;
+	size_t length;
+	int enumtag;
+} piv_cache_item;
+
+typedef struct piv_private_data {
 	struct sc_pin_cmd_pin pin_info;
 	sc_file_t *aid_file;
 	int enumtag;
@@ -39,8 +51,63 @@ struct piv_private_data {
 	size_t max_recv_size; /* saved size, need to lie to pkcs15_read_file */
 	size_t max_send_size; 
 	int key_ref; /* saved from set_security_env and */
-	int alg_id;  /* used in decrypy, signature */ 
-};
+	int alg_id;  /* used in decrypt, signature */ 
+	piv_cache_item* cache;
+	int cacheLen;
+	piv_cache_item* current_item;
+} piv_private_data_t;
+
+#define PIV_DATA(card) ((piv_private_data_t*)card->drv_data)
+
+static int add_cache_item(piv_private_data_t* priv, int enumtag, u8* data, size_t length) {
+	int idx, len = priv->cacheLen;
+	piv_cache_item* cache = priv->cache;
+	for(idx = 0; idx < len; idx++) {
+		if(!cache[idx].data)
+			break;
+		if(cache[idx].enumtag == enumtag) /* Found matching tag */
+			break;
+	}
+	if(idx == len) 
+		return -1; /* FAILED NO FREE ROOM */
+	if(cache[idx].data)
+		free(cache[idx].data);
+	cache[idx].data = data;
+	cache[idx].length = length;
+	cache[idx].enumtag = enumtag;
+	return 0;
+}
+
+static piv_cache_item* get_cache_item(piv_private_data_t* priv, int enumtag) {
+	int idx, len = priv->cacheLen;
+	piv_cache_item* cache = priv->cache;
+#if 1
+/* DEE some thing not right with the cache,
+ * when used with the pkcs15-tool -C it cant find the CCC, but finds the
+ * CHUI ( as it was read for the serial) and finds part of a cert for the 
+ * CHFingerprints
+ * So turn it off for now. 6/1/2007
+ */
+return NULL; 
+#endif
+
+	for(idx = 0; idx < len; idx++) {
+		if(cache[idx].enumtag == enumtag)
+			return &cache[idx];
+	}
+	return NULL;
+}
+
+static void free_cache_items(piv_private_data_t* priv) {
+	int idx, len = priv->cacheLen;
+	piv_cache_item* cache = priv->cache;
+	for(idx = 0; idx < len; idx++) {
+		if(cache[idx].data) {
+			free(cache[idx].data);
+			cache[idx].data = NULL;
+		}
+	}
+}
 
 struct piv_aid {
 	int enumtag;
@@ -71,7 +138,11 @@ enum {
 	PIV_OBJ_X509_CARD_AUTH,
 	PIV_OBJ_SEC_OBJ,
 	PIV_OBJ_9B03,
-	PIV_OBJ_9A06
+	PIV_OBJ_9A06,
+	PIV_OBJ_9C06,
+	PIV_OBJ_9D06,
+	PIV_OBJ_9E06,
+	PIV_CACHE_SIZE
 };
 
 struct piv_object {
@@ -88,14 +159,12 @@ static struct piv_object piv_objects[] = {
 	{ PIV_OBJ_CCC, "Card Capability Container", 
 			"2.16.840.1.101.3.7.1.219.0", 3, "\x5F\xC1\x07", "\xDB\x00", 266},
 	{ PIV_OBJ_CHUI, "Card Holder Unique Identifier", 
-			"2.16.840.1.101.3.7.2.48.0", 3, "\x5F\xC1\x02", "\x30\x00", 3377},
+			"2.16.840.1.101.3.7.2.48.0", 3, "\x5F\xC1\x02", "\x30\x00", 3379}, /* Updated per SP800-73-1 Errata */
 	{ PIV_OBJ_X509_PIV_AUTH, "X.509 Certificate for PIV Authentication", 
 			"2.16.840.1.101.3.7.2.1.1", 3, "\x5F\xC1\x05", "\x01\x01", 1856+4+400} , 
 		/* extra 400 is hack for MultOS card which returns 2200 bytes  */
-	{ PIV_OBJ_CHF1, "Card Holder Fingerprint I",
-			"2.16.840.1.101.3.7.2.96.16", 3, "\x5F\xC1\x03", "\x60\x01", 7768},
-	{ PIV_OBJ_CHF2, "Card Holder Fingerprint II",
-			"2.16.840.1.101.3.7.2.96.17", 3, "\x5F\xC1\x04", "\x60\x11", 7768},
+	{ PIV_OBJ_CHF1, "Card Holder Fingerprints",
+			"2.16.840.1.101.3.7.2.96.16", 3, "\x5F\xC1\x03", "\x60\x10", 7768},
 	{ PIV_OBJ_PI, "Printed Information", 
 			"2.16.840.1.101.3.7.2.48.1", 3, "\x5F\xC1\x09", "\x30\x01", 106},
 	{ PIV_OBJ_CHFI, "Card Holder Facial Image",
@@ -111,9 +180,19 @@ static struct piv_object piv_objects[] = {
 /* following not standard , to be used by piv-tool only for testing */
 	{ PIV_OBJ_9B03, "3DES-ECB ADM", 
 			"2.16.840.1.101.3.7.2.9999.3", 2, "\x9B\x03", "\x9B\x03", 24},
-	{ PIV_OBJ_9A06, "Pub key from last genkey",
-			"2.16.840.1.101.3.7.2.9999.20", 2, "\x9A\x06", "\x9A\x06", 255},
-	/* need other RSA types */
+	/* Only used when signing a cert req, usually from engine
+	 * after piv-tool generated the key and saved the pub key 
+	 * to a file. Note RSA key can be 1024, 2048 or 3072 
+	 * but still use the "9x06" name.
+	 */
+	{ PIV_OBJ_9A06, "RSA 9A Pub key from last genkey",
+			"2.16.840.1.101.3.7.2.9999.20", 2, "\x9A\x06", "\x9A\x06", 512},
+	{ PIV_OBJ_9C06, "Pub 9C key from last genkey",
+			"2.16.840.1.101.3.7.2.9999.21", 2, "\x9C\x06", "\x9C\x06", 512},
+	{ PIV_OBJ_9D06, "Pub 9D key from last genkey",
+			"2.16.840.1.101.3.7.2.9999.22", 2, "\x9D\x06", "\x9D\x06", 512},
+	{ PIV_OBJ_9E06, "Pub 9E key from last genkey",
+			"2.16.840.1.101.3.7.2.9999.23", 2, "\x9E\x06", "\x9E\x06", 512},
 	{ 0, "", "", 0, "", "", 0}
 };
 	
@@ -168,17 +247,6 @@ static size_t put_tag_and_len(unsigned int tag, size_t len, u8 **ptr)
 	return i;
 }
 
-
-static int piv_logout(sc_card_t * card)
-{
-	SC_FUNC_CALLED(card->ctx,1);
-	/* 
-	 * nothing to do here, as we dont have files on the card.  
-	 */ 
-	return 0;
-}
-
-
 /*
  * Send a command and receive data. Receive as much as the card indicates 
  * in the first segment. There is always something to send. 
@@ -195,7 +263,7 @@ static int piv_general_io(sc_card_t *card, int ins, int p1, int p2,
 	const u8 * sendbuf, size_t sendbuflen, u8 ** recvbuf,
 	size_t * recvbuflen)
 {
-	struct piv_private_data * priv = (struct piv_private_data *) card->drv_data;
+	piv_private_data_t * priv = PIV_DATA(card);
 	int r;
 	sc_apdu_t apdu;
 	u8 rbufinitbuf[20000]; /* crude way to do this  see above comments */
@@ -216,7 +284,7 @@ static int piv_general_io(sc_card_t *card, int ins, int p1, int p2,
 
 	r = sc_lock(card);
 	if (r != SC_SUCCESS)
-		return r;
+		SC_FUNC_RETURN(card->ctx, 1, r);
 		
 	sc_format_apdu(card, &apdu, 
 			recvbuf ? SC_APDU_CASE_4_SHORT: SC_APDU_CASE_3_SHORT, 
@@ -246,7 +314,7 @@ static int piv_general_io(sc_card_t *card, int ins, int p1, int p2,
 	r = sc_transmit_apdu(card, &apdu);
 	card->max_recv_size = 0xffff;
 
-	sc_debug(card->ctx,"DEE  r=%d apdu.resplen=%d sw1=%02x sw2=%02x", 
+	sc_debug(card->ctx,"DEE r=%d apdu.resplen=%d sw1=%02x sw2=%02x", 
 			r, apdu.resplen, apdu.sw1, apdu.sw2);
 	if (r < 0) {
 		sc_debug(card->ctx,"Transmit failed");
@@ -261,11 +329,12 @@ static int piv_general_io(sc_card_t *card, int ins, int p1, int p2,
 	}
 
 	/*
-	 * See how much we read and if we need to read more 
-	 * We should have read a tag, which will tell up how big of 
-	 * a buffer we need. We will then read more in to the allocated buffer 
+	 * See how much we read and make sure it is asn1 
+	 * if not, return 0 indicating no data found
 	 */  
 
+	
+	rbuflen = 0;  /* in case rseplen < 3  i.e. not parseable */
 	if ( recvbuflen && recvbuf && apdu.resplen > 3) {
 		*recvbuflen = 0;
 		/* we should have all the tag data, so we have to tell sc_asn1_find_tag 
@@ -398,6 +467,30 @@ err:
 }
 
 
+static int piv_select_aid(sc_card_t* card, u8* aid, size_t aidlen, u8* response, size_t *responselen)
+{
+	sc_apdu_t apdu;
+	int r;
+
+	SC_FUNC_CALLED(card->ctx,4);
+	if (card->ctx->debug >= 5)
+		sc_debug(card->ctx, "Got args: aid=%x, aidlen=%d, response=%x, responselen=%d\n", aid, aidlen, response, *responselen);
+
+	sc_format_apdu(card, &apdu, 
+		response == NULL ? SC_APDU_CASE_3_SHORT : SC_APDU_CASE_4_SHORT, 0xA4, 0x04, 0x00);
+	apdu.lc = aidlen;
+	apdu.data = aid;
+	apdu.datalen = aidlen;
+	apdu.resp = response;
+	apdu.resplen = *responselen;
+	apdu.le = response == NULL ? 0 : 256; /* could be 21  for fci */
+
+	r = sc_transmit_apdu(card, &apdu);
+	*responselen = apdu.resplen;
+	SC_TEST_RET(card->ctx, 4,  r);
+	SC_FUNC_RETURN(card->ctx, 4,  sc_check_sw(card, apdu.sw1, apdu.sw2));
+}
+
 /* find the PIV AID on the card. If card->type already filled in,
  * then look for specific AID only
  * Assumes that priv may not be present
@@ -412,6 +505,7 @@ static int piv_find_aid(sc_card_t * card, sc_file_t *aid_file)
 	size_t taglen;
 	u8 *pix;
 	size_t pixlen;
+	size_t resplen = sizeof(rbuf);
 
 	SC_FUNC_CALLED(card->ctx,1);
 
@@ -422,19 +516,9 @@ static int piv_find_aid(sc_card_t * card, sc_file_t *aid_file)
 	if (card->type == SC_CARD_TYPE_PIV_II_GENERIC) 
 		SC_FUNC_RETURN(card->ctx, 1, 0);
 
-	sc_format_apdu(card, &apdu, SC_APDU_CASE_4_SHORT, 0xA4, 0x04, 0x00);
-	apdu.lc = piv_aids[0].len_short;
-	apdu.data = piv_aids[0].value;
-	apdu.datalen = piv_aids[0].len_short;
-	apdu.resp = rbuf;
-	apdu.resplen = sizeof(rbuf);
-	apdu.le = 256; /* could be 21  for fci */
-
-	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
-	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
-	if (r >= 0 && apdu.resplen > 2 ) {
-		tag = (u8 *) sc_asn1_find_tag(card->ctx, rbuf, apdu.resplen, 0x61, &taglen);
+	r = piv_select_aid(card, piv_aids[0].value, piv_aids[0].len_short, rbuf, &resplen);
+	if (r >= 0 && resplen > 2 ) {
+		tag = (u8 *) sc_asn1_find_tag(card->ctx, rbuf, resplen, 0x61, &taglen);
 		if (tag != NULL) {
 			pix = (u8 *) sc_asn1_find_tag(card->ctx, tag, taglen, 0x4F, &pixlen);
 			if (pix != NULL ) { 
@@ -513,12 +597,13 @@ static int piv_find_aid(sc_card_t * card, sc_file_t *aid_file)
 static int piv_get_data(sc_card_t * card, unsigned int enumtag, 
 			u8 **buf, size_t *buf_len)
 {
-	struct piv_private_data * priv = (struct piv_private_data *) card->drv_data;
+	piv_private_data_t * priv = PIV_DATA(card);
 	u8 *p;
 	int r = 0;
 	u8 tagbuf[8];
 	size_t tag_len;
-		
+	char * keyenvname = NULL;
+	
 	SC_FUNC_CALLED(card->ctx,1);
 	sc_debug(card->ctx, "get_data: tag=%d \n", enumtag);
 
@@ -536,11 +621,26 @@ static int piv_get_data(sc_card_t * card, unsigned int enumtag,
 	 * as an OpenSSL EVP_KEY PEM using the -o parameter
 	 * we will look to see there as a file and load it
 	 * this is ugly, and maybe the pkcs15 cache would work
-	 * but we only need it  to get the OpenSSL req with engine to work.
+	 * but we only need it to get the OpenSSL req with engine to work.
+	 * Each of the 3 keys with certs has its own file. 
 	 */
 
+	switch (piv_objects[enumtag].enumtag) {
+		case PIV_OBJ_9A06:
+			keyenvname = "PIV_9A06_KEY";
+			break;
+		case PIV_OBJ_9C06:
+			keyenvname = "PIV_9C06_KEY";
+			break;
+		case PIV_OBJ_9D06:
+			keyenvname = "PIV_9D06_KEY";
+			break;
+		case PIV_OBJ_9E06:
+			keyenvname = "PIV_9E06_KEY";
+			break;
+	}
 
-	if (piv_objects[enumtag].enumtag == PIV_OBJ_9A06)  {
+	if (keyenvname)  {
 		BIO * bp = NULL;
 		RSA * rsa = NULL;
 		u8 *q;
@@ -548,7 +648,7 @@ static int piv_get_data(sc_card_t * card, unsigned int enumtag,
 		size_t taglen;
 		char * keyfilename = NULL;
 
-		keyfilename = getenv("PIV_9A06_KEY");
+		keyfilename = getenv(keyenvname);
 
 		if (keyfilename == NULL) {
 			r = SC_ERROR_FILE_NOT_FOUND;
@@ -610,7 +710,105 @@ err:
 
 	SC_FUNC_RETURN(card->ctx, 1, r);
 }
-	  
+
+
+static int piv_handle_certificate_data(sc_card_t *card,
+		int enumtag,
+		unsigned idx, u8* buf, size_t count,
+		u8* data, size_t length) {
+	piv_private_data_t * priv = PIV_DATA(card);
+	u8* tag;
+	size_t taglen;
+	int compressed = 0;
+	piv_cache_item* item;
+	/* get the certificate out */
+	tag = (u8 *) sc_asn1_find_tag(card->ctx, data, length, 0x71, &taglen);
+	if (tag && (((*tag) & 0x80) || ((*tag) & 0x01))) {
+		compressed = 1;
+	}
+	tag = (u8 *) sc_asn1_find_tag(card->ctx, data,  length, 0x70, &taglen);
+	if (tag == NULL) {
+	SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_OBJECT_NOT_VALID);
+	}
+	/* Potential truncation */
+	if(compressed) {
+#ifdef HAVE_ZLIB_H
+		size_t len = count;
+		u8* newBuf = NULL;
+		if(SC_SUCCESS != sc_decompress_alloc(&newBuf, &len, tag, taglen, COMPRESSION_AUTO)) {
+			SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_OBJECT_NOT_VALID);
+		} else {
+			if(len < count + idx)
+				count = len - idx;
+			if(count <= 0)
+				SC_FUNC_RETURN(card->ctx, 1, 0);
+			memcpy(buf, newBuf + idx, count);
+			if(0 != add_cache_item(priv, enumtag, newBuf, len)) {
+				/* Failed to cache item */
+				free(newBuf);
+			}
+		}
+		SC_FUNC_RETURN(card->ctx, 1, count);
+#else
+		sc_error(card->ctx,"PIV compression not supported, no zlib");
+		SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_NOT_SUPPORTED);
+#endif
+	}
+	if(taglen < count + idx)
+		count = taglen - idx;
+	if(count <= 0)
+		SC_FUNC_RETURN(card->ctx, 1, 0);
+	memcpy(buf, tag, count);
+	if(0 == add_cache_item(priv, enumtag, NULL, 0)) {
+		/* Space available in the cache array, use it */
+		item = get_cache_item(priv, enumtag);
+		if(item) {
+			item->data = malloc(taglen);
+			if(item->data) {
+				item->length = taglen;
+				memcpy(item->data, tag, taglen);
+			}
+		}
+	}
+	SC_FUNC_RETURN(card->ctx, 1, count);
+}
+
+static int piv_handle_data(sc_card_t *card, int enumtag, 
+		unsigned idx, u8* buf, size_t count, u8* data, size_t length) {
+	/* We need to return the whole not just first tag */
+	piv_private_data_t * priv = PIV_DATA(card);
+	u8* tag;
+	size_t taglen;
+	size_t reallen;
+	piv_cache_item* item;
+	
+	SC_FUNC_CALLED(card->ctx,1);
+
+	tag = (u8 *) sc_asn1_find_tag(card->ctx, data,  length, *data, &taglen);
+	if (tag == NULL) {
+		SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_OBJECT_NOT_VALID);
+	}
+	reallen = tag - data + taglen;
+	if (reallen > length)
+		SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_OBJECT_NOT_VALID);
+
+	if(reallen < count + idx)
+		count = reallen - idx;
+	if(count <= 0)
+		return 0;
+	memcpy(buf, data + idx, count);
+	if(0 == add_cache_item(priv, enumtag, NULL, 0)) {
+		item = get_cache_item(priv, enumtag);
+		if(item) {
+			item->data = malloc(reallen);
+			if(item->data) {
+				item->length = reallen;
+				memcpy(item->data, data, reallen);
+			}
+		}
+	}
+	return count;
+}
 /* 
  * Callers of this are expecting a file without tags,
  * So we need to know what type of file this is, so we can get the
@@ -620,7 +818,9 @@ err:
 static int piv_read_binary(sc_card_t *card, unsigned int idx,
 		unsigned char *buf, size_t count, unsigned long flags)
 {
-	struct piv_private_data * priv = (struct piv_private_data *) card->drv_data;
+	piv_private_data_t * priv = PIV_DATA(card);
+	piv_cache_item* item = NULL;
+	int enumtag;
 	int r;
 	u8 *rbuf = NULL;
 	size_t rbuflen = 0;
@@ -632,15 +832,31 @@ static int piv_read_binary(sc_card_t *card, unsigned int idx,
 	SC_FUNC_CALLED(card->ctx,1);
 	if (priv->selected_obj < 0) 
 		 SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_INTERNAL);
+	enumtag = piv_objects[priv->selected_obj].enumtag;
 
 	if (priv->eof == 1)
 		return 0;
 
+	/* Hit the cache */
+	if(priv->current_item) {
+		item = priv->current_item;
+		if(idx + count > item->length) {
+			count = item->length - idx;
+		}
+		if(count <= 0) {
+			priv->eof = 1;
+			return 0;
+		}
+		memcpy(buf, item->data + idx, count);
+		return count;
+	}
+
+
 	r = piv_get_data(card, priv->selected_obj, &rbuf, &rbuflen);
 	
 	if (r >=0) {
 		/* if tag is 0, assume card is telling us no object on card */
-		if (rbuf[0] == '0') {
+		if (!rbuf || rbuf[0] == 0x00 || (rbuf[0] == 0x53 && rbuf[1] == 0x00)) {
 			r = SC_ERROR_FILE_NOT_FOUND;
 			goto err;
 		}
@@ -650,34 +866,28 @@ static int piv_read_binary(sc_card_t *card, unsigned int idx,
 			/* if missing, assume its the body */
 			/* DEE bug in the beta card */
 			sc_debug(card->ctx," ***** tag 0x53 MISSING \n");
-			body =  rbuf;
-			bodylen = rbuflen;
-#if 0
 			r = SC_ERROR_INVALID_DATA;
 			goto err;
-#endif
 		}
-		switch (piv_objects[priv->selected_obj].enumtag) {
+		if (bodylen > body - rbuf + rbuflen) {
+			sc_debug(card->ctx," ***** tag length > then data: %d>%d+%d",
+				bodylen , body - rbuf, rbuflen);
+			r = SC_ERROR_INVALID_DATA;
+			goto err;
+		}
+		switch (enumtag) {
 			case PIV_OBJ_X509_PIV_AUTH:
 			case PIV_OBJ_X509_DS:
 			case PIV_OBJ_X509_KM:
 			case PIV_OBJ_X509_CARD_AUTH:
-				/* get the certificate out */
-				tag = (u8 *) sc_asn1_find_tag(card->ctx, body,  bodylen, 0x70, &taglen);
-				if (tag == NULL) {
-					r = SC_ERROR_OBJECT_NOT_VALID;
-					break;
-				}
-				memcpy(buf, tag, taglen); /*DEE should check lengths */
-				r = taglen;
-				tag = (u8 *) sc_asn1_find_tag(card->ctx, body,  bodylen, 0x71, &taglen);
-				if (tag && ((*tag) & 0x80)) {
-					sc_debug(card->ctx,0,"Cert is gziped! %0x2.2x\n",*tag);
-				}
+				r = piv_handle_certificate_data(card, enumtag, idx, buf, count, body, bodylen);
 				break;
-			default:
-				/* For now get the first tag, and return the data */
-				tag = (u8 *) sc_asn1_find_tag(card->ctx, body,  bodylen, *body, &taglen);
+			case PIV_OBJ_9A06:
+			case PIV_OBJ_9C06:
+			case PIV_OBJ_9D06:
+			case PIV_OBJ_9E06:
+				tag = (u8 *) sc_asn1_find_tag(card->ctx, body,  bodylen,
+						*body, &taglen);
 				if (tag == NULL) {
 					r = SC_ERROR_OBJECT_NOT_VALID;
 					break;
@@ -685,6 +895,11 @@ static int piv_read_binary(sc_card_t *card, unsigned int idx,
 				memcpy(buf, tag, taglen); /*DEE should check lengths */
 				r = taglen;
 				break;
+			
+			default:
+				/* send whole object, tags and all */
+				r = piv_handle_data(card, enumtag, idx, buf, count, rbuf, rbuflen);
+				break;
 		}
 	}
 err:
@@ -713,7 +928,7 @@ static int piv_put_data(sc_card_t *card, unsigned int tag,
 	tag_len = piv_objects[tag].tag_len;
 	sbuflen = put_tag_and_len(0x5c, tag_len, NULL) + buf_len;
 	if (!(sbuf = (u8 *) malloc(sbuflen))) 
-		return SC_ERROR_OUT_OF_MEMORY;
+		SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_OUT_OF_MEMORY);
 
 	p = sbuf;
 	put_tag_and_len(0x5c, tag_len, &p);
@@ -731,23 +946,57 @@ static int piv_put_data(sc_card_t *card, unsigned int tag,
 	SC_FUNC_RETURN(card->ctx, 1, r);
 }
 
+static int piv_write_certificate(sc_card_t *card,
+		unsigned idx, const u8* buf, size_t count,
+		unsigned long flags) {
+	piv_private_data_t * priv = PIV_DATA(card);
+	int r = SC_SUCCESS;
+	u8 *sbuf = NULL;
+	u8 *p;
+	size_t sbuflen;
+	size_t taglen;
 
+	sc_debug(card->ctx,"DEE cert len=%d",count);
+	taglen = put_tag_and_len(0x70, count, NULL) 
+		+ put_tag_and_len(0x71, 1, NULL)
+		+ put_tag_and_len(0xFE, 0, NULL);
+
+	sbuflen =  put_tag_and_len(0x53, taglen, NULL);
+
+	sbuf = (u8*) malloc(sbuflen);
+	if (sbuf == NULL) 
+		SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_OUT_OF_MEMORY);
+	p = sbuf;
+	put_tag_and_len(0x53, taglen, &p);
+
+	put_tag_and_len(0x70, count, &p);
+	memcpy(p, buf, count);
+	p += count;
+	put_tag_and_len(0x71, 1, &p);
+	*p++ = (flags && 1)? 0x80:0x00; /* certinfo, i.e. gziped? */
+	put_tag_and_len(0xFE,0,&p); /* LRC tag */
+
+	sc_debug(card->ctx,"DEE buf %p len %d %d", sbuf, p -sbuf, sbuflen);
+
+	r = piv_put_data(card, priv->selected_obj, sbuf, sbuflen);
+	if (sbuf)
+		free(sbuf);
+
+	SC_FUNC_RETURN(card->ctx, 1, r);
+}
 /* 
  * We need to add the 0x53 tag and other specific tags, 
  * and call the piv_put_data 
  * Note: the select file will have saved the object type for us 
+ * Write is only used by piv-tool, so we will use flags==1
+ * to indicate we are writing a compressed cert. 
  */
 
 static int piv_write_binary(sc_card_t *card, unsigned int idx,
 		const u8 *buf, size_t count, unsigned long flags)
 {
-	struct piv_private_data * priv = (struct piv_private_data *) card->drv_data;
-	int r = 0;
-	u8 *sbuf = NULL;
-	u8 *p;
-	size_t sbuflen;
-	size_t taglen;
-	
+	piv_private_data_t * priv = PIV_DATA(card);
+	/* TODO: Add cache hooks */	
 	SC_FUNC_CALLED(card->ctx,1);
 
 	if (priv->selected_obj < 0)
@@ -760,40 +1009,12 @@ static int piv_write_binary(sc_card_t *card, unsigned int idx,
 		case PIV_OBJ_X509_DS:
 		case PIV_OBJ_X509_KM:
 		case PIV_OBJ_X509_CARD_AUTH:
-		sc_debug(card->ctx,"DEE cert len=%d",count);
-		
-			taglen = put_tag_and_len(0x70, count, NULL) 
-				+ put_tag_and_len(0x71, 1, NULL);
-
-			sbuflen =  put_tag_and_len(0x53, taglen, NULL);
-
-			sbuf = (u8*) malloc(sbuflen);
-			if (sbuf == NULL) 
-				SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_OUT_OF_MEMORY);
-			p = sbuf;
-			put_tag_and_len(0x53, taglen, &p);
-
-			put_tag_and_len(0x70, count, &p);
-			memcpy(p, buf, count);
-			p += count;
-			put_tag_and_len(0x71, 1, &p);
-			*p++ = 0x00; /* certinfo, i.e. not gziped */
-			sc_debug(card->ctx,"DEE buf %p len %d %d", sbuf, p -sbuf, sbuflen);
-			break;
-			
+			SC_FUNC_RETURN(card->ctx, 1, piv_write_certificate(card, idx, buf, count, flags));
 		default:
 		sc_debug(card->ctx, "Don't know how to write object %s\n",
 			piv_objects[priv->selected_obj].name);
-			r = SC_ERROR_NOT_SUPPORTED;
-			break;
+			SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_NOT_SUPPORTED);
 	}
-
-	if (r == 0) 
-		r = piv_put_data(card, priv->selected_obj, sbuf, sbuflen);
-	if (sbuf)
-		free(sbuf);
-
-	SC_FUNC_RETURN(card->ctx, 1, r);
 }
 
 /*
@@ -803,8 +1024,7 @@ static int piv_write_binary(sc_card_t *card, unsigned int idx,
  * This is only needed during initialization/personalization of the card 
  */
 
-static int piv_get_3des_key(sc_card_t *card, 
-		DES_cblock *k1, DES_cblock *k2,  DES_cblock *k3)
+static int piv_get_3des_key(sc_card_t *card, u8 *key)
 {
 
 	int r;
@@ -837,13 +1057,13 @@ static int piv_get_3des_key(sc_card_t *card,
 	keybuf[23] = '\0';
 	keybuf[47] = '\0';
 	outlen = 8;
-	r = sc_hex_to_bin(keybuf, *k1, &outlen);
+	r = sc_hex_to_bin(keybuf, key, &outlen);
 	if (r) goto err;
 	outlen = 8;
-	r = sc_hex_to_bin(keybuf+24, *k2, &outlen);
+	r = sc_hex_to_bin(keybuf+24, key+8, &outlen);
 	if (r) goto err;
 	outlen = 8;
-	r = sc_hex_to_bin(keybuf+48, *k3, &outlen);
+	r = sc_hex_to_bin(keybuf+48, key+16, &outlen);
 	if (r) goto err;
 	
 err:
@@ -862,19 +1082,19 @@ static int piv_general_mutual_authenticate(sc_card_t *card,
 {
 	int r;
 	size_t N;
-	int locked = 0;
+	int locked = 0, outl;
 	u8  *rbuf = NULL;
 	size_t rbuflen;
 	u8 nonce[8] = {0xDE, 0xE0, 0xDE, 0xE1, 0xDE, 0xE2, 0xDE, 0xE3};
-	u8 sbuf[255];
+	u8 sbuf[255], key[24];
 	u8 *p, *q;
-	
-	DES_cblock k1, k2, k3;
-	DES_key_schedule ks1, ks2, ks3;
+	EVP_CIPHER_CTX ctx;
 
 	SC_FUNC_CALLED(card->ctx,1);
 
-	r = piv_get_3des_key(card, &k1, &k2, &k3);
+	EVP_CIPHER_CTX_init(&ctx);
+
+	r = piv_get_3des_key(card, key);
 	if (r != SC_SUCCESS)
 		goto err;
 
@@ -913,11 +1133,12 @@ static int piv_general_mutual_authenticate(sc_card_t *card,
 	*p++ = 0x80;
 	*p++ = (u8)N; 
 	
-	DES_set_key((DES_cblock *) &k1, &ks1);
-	DES_set_key((DES_cblock *) &k2, &ks2);
-	DES_set_key((DES_cblock *) &k3, &ks3);
-	/* FIXME */
-	DES_ecb3_encrypt((const_DES_cblock *)q, (DES_cblock *)p, &ks1, &ks2, &ks3, DES_DECRYPT);
+	EVP_DecryptInit_ex(&ctx, EVP_des_ede3(), NULL, key, NULL);
+	if (!EVP_DecryptUpdate(&ctx, p, &outl, q, 8)) {
+		r = SC_ERROR_INTERNAL;
+		goto err;
+	}
+	
 	p += N;
 
 	*p++ = 0x81;
@@ -939,22 +1160,21 @@ static int piv_general_mutual_authenticate(sc_card_t *card,
 		goto err;
 	}
 	
-	DES_set_key((DES_cblock *) &k1, &ks1);
-	DES_set_key((DES_cblock *) &k2, &ks2);
-	DES_set_key((DES_cblock *) &k3, &ks3);
-	
 	p = sbuf;
-	/* FIXME */
-	DES_ecb3_encrypt((const_DES_cblock *)q, (DES_cblock *)p, &ks1, &ks2, &ks3, DES_DECRYPT);
+	if (!EVP_DecryptUpdate(&ctx, p, &outl, q, 8)) {
+		r = SC_ERROR_INTERNAL;
+		goto err;
+	}
 	
 	if (memcmp(nonce, p, N) != 0) {
 		sc_debug(card->ctx, "mutual authentication failed, card returned wrong value");
 		r = SC_ERROR_DECRYPT_FAILED;
 		goto err;
 	}
-	r = 0;
+	r = SC_SUCCESS;
 
 err:
+	EVP_CIPHER_CTX_cleanup(&ctx);
 	if (locked) 
 		sc_unlock(card);
 	if (rbuf)
@@ -966,19 +1186,19 @@ err:
 static int piv_general_external_authenticate(sc_card_t *card, 
 		unsigned int key_ref, unsigned int alg_id)
 {
-	struct piv_private_data * priv = (struct piv_private_data *) card->drv_data;
-	int r;
+	/* unused: piv_private_data_t * priv = PIV_DATA(card); */
+	int r, outl;
 	u8  *rbuf = NULL;
 	size_t rbuflen;
-	u8 sbuf[255];
+	u8 sbuf[255], key[24];
 	u8 *p, *q;
+	EVP_CIPHER_CTX ctx;
 	
-	DES_cblock k1, k2, k3;
-	DES_key_schedule ks1, ks2, ks3;
-
 	SC_FUNC_CALLED(card->ctx,1);
 
-	r = piv_get_3des_key(card, &k1, &k2, &k3);
+	EVP_CIPHER_CTX_init(&ctx);
+
+	r = piv_get_3des_key(card, key);
 	if (r != SC_SUCCESS)
 		goto err;
 
@@ -1013,29 +1233,63 @@ static int piv_general_external_authenticate(sc_card_t *card,
 	*p++ = *(rbuf + 1);
 	*p++ = 0x82;
 	*p++ = *(rbuf + 3);
-	
-	DES_set_key((DES_cblock *) &k1, &ks1);
-	DES_set_key((DES_cblock *) &k2, &ks2);
-	DES_set_key((DES_cblock *) &k3, &ks3);
-	
-	DES_ecb3_encrypt((const_DES_cblock *)q, (DES_cblock *)p, &ks1, &ks2, &ks3, DES_ENCRYPT);
+
+	EVP_EncryptInit_ex(&ctx, EVP_des_ede3(), NULL, key, NULL);
+	if (!EVP_EncryptUpdate(&ctx, p, &outl, q, 8)) {
+		r = SC_ERROR_INTERNAL;
+		goto err;
+	}	
 	p += *(rbuf + 3);
 	
-	switch (priv-> enumtag) {
-		default:
-			alg_id = 0x00;
-	}
 	r = piv_general_io(card, 0x87, alg_id, key_ref, sbuf, p - sbuf, NULL, NULL);
 
 	sc_unlock(card);
 
 err:
+	EVP_CIPHER_CTX_cleanup(&ctx);
+	sc_mem_clear(key, sizeof(key));
 	if (rbuf)
 		free(rbuf);
 
 	SC_FUNC_RETURN(card->ctx, 1, r);
 }
 
+static int piv_get_serial_nr_from_CHUI(sc_card_t* card, sc_serial_number_t* serial)
+{
+	int r;
+	u8 *rbuf = NULL;
+	u8 *body, *fascn;
+	size_t rbuflen = 0, bodylen, fascnlen;
+	u8 temp[2000];
+	size_t templen = sizeof(temp);
+
+	SC_FUNC_CALLED(card->ctx, 1);
+
+	/* ensure we've got the PIV selected, and nothing else is in process */
+	/* This fixes several problems due to previous incomplete APDUs during card detection */
+	/* Note: We need the temp because (some?) Oberthur cards don't like selecting an applet without response data */
+	piv_select_aid(card, piv_aids[0].value, piv_aids[0].len_short, temp, &templen);
+
+	r = piv_get_data(card, PIV_OBJ_CHUI-1, &rbuf, &rbuflen);
+	SC_TEST_RET(card->ctx, r, "Failure retrieving CHUI");
+
+	r = SC_ERROR_INTERNAL;
+	if (rbuflen != 0) {
+		body = (u8 *)sc_asn1_find_tag(card->ctx, rbuf, rbuflen, 0x53, &bodylen); /* Pass the outer wrapper asn1 */
+		if (body != NULL && bodylen != 0) {
+			fascn = (u8 *)sc_asn1_find_tag(card->ctx, body, bodylen, 0x30, &fascnlen); /* Find the FASC-N data */
+			if (fascn != NULL && fascnlen != 0) {
+				serial->len = fascnlen < SC_MAX_SERIALNR ? fascnlen : SC_MAX_SERIALNR;
+				memcpy (serial->value, fascn, serial->len);
+				r = SC_SUCCESS;
+			}
+		}
+	}
+      
+	if (rbuf != NULL)
+		free (rbuf);
+	SC_FUNC_RETURN(card->ctx, 1, r);
+}
 
 static int piv_card_ctl(sc_card_t *card, unsigned long cmd, void *ptr)
 {
@@ -1060,6 +1314,9 @@ static int piv_card_ctl(sc_card_t *card, unsigned long cmd, void *ptr)
 			return piv_generate_key(card, 
 				(struct sc_cardctl_cryptoflex_genkey_info *) ptr);
 			break;
+		case SC_CARDCTL_GET_SERIALNR:
+			return piv_get_serial_nr_from_CHUI(card, (sc_serial_number_t *) ptr);
+			break;
 	}
 
 	return SC_ERROR_NOT_SUPPORTED;
@@ -1092,14 +1349,17 @@ static int piv_get_challenge(sc_card_t *card, u8 *rnd, size_t len)
 
 		r = piv_general_io(card, 0x87, 0x00, 0x00, sbuf, p - sbuf, 
 				&rbuf, &rbuflen); 
- 		if (r < 0) 
+ 		if (r < 0) { 
+			sc_unlock(card);
 			SC_FUNC_RETURN(card->ctx, 1, r);
+		}
 		q = rbuf;
 		if ( (*q++ != 0x7C)
 			|| (*q++ != rbuflen - 2)
 			|| (*q++ != 0x81)
 			|| (*q++ != rbuflen - 4)) {
 			r =  SC_ERROR_INVALID_DATA;
+			sc_unlock(card);
 			SC_FUNC_RETURN(card->ctx, 1, r);
 		}
 		memcpy(rnd, q, n);
@@ -1119,7 +1379,7 @@ static int piv_set_security_env(sc_card_t *card,
                     const sc_security_env_t *env,
                     int se_num)
 {
-	struct piv_private_data * priv = (struct piv_private_data *) card->drv_data;
+	piv_private_data_t * priv = PIV_DATA(card);
 	
 	SC_FUNC_CALLED(card->ctx,1);
 
@@ -1127,13 +1387,13 @@ static int piv_set_security_env(sc_card_t *card,
 			env->flags, env->operation, env->algorithm, env->algorithm_flags, 
 			env->algorithm_ref, env->key_ref[0], env->key_ref_len);
 
-	if (env->algorithm == 0) 
-		priv->alg_id = 0x06; /* 1024 RSA for now only */
+	if (env->algorithm == SC_ALGORITHM_RSA) 
+		priv->alg_id = 0x06; /* Say it is RSA, set 5, 6, 7 later */
 	else
 		SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_NO_CARD_SUPPORT);
 	priv->key_ref = env->key_ref[0];
 
-	return 0;
+	SC_FUNC_RETURN(card->ctx, 2, 0);
 }
 
 
@@ -1141,7 +1401,7 @@ static int piv_restore_security_env(sc_card_t *card, int se_num)
 {
 	SC_FUNC_CALLED(card->ctx,1);
 
-	return 0;
+	SC_FUNC_RETURN(card->ctx, 1, 0);
 }
 
 
@@ -1149,15 +1409,16 @@ static int piv_validate_general_authentication(sc_card_t *card,
 					const u8 * data, size_t datalen,
 					u8 * out, size_t outlen)
 {
-	struct piv_private_data * priv = (struct piv_private_data *) card->drv_data;
+	piv_private_data_t * priv = PIV_DATA(card);
 	int r;
 	u8 *p;
 	u8 *tag;
 	size_t taglen;
 	u8 *body;
 	size_t bodylen;
+	unsigned int real_alg_id;
 
-	u8 sbuf[SC_MAX_APDU_BUFFER_SIZE];
+	u8 sbuf[4096]; /* needs work. for 3072 keys, needs 384+10 or so */
 	u8 *rbuf = NULL;
 	size_t rbuflen;
 	
@@ -1172,7 +1433,25 @@ static int piv_validate_general_authentication(sc_card_t *card,
 	memcpy(p, data, datalen);
 	p += datalen;
 
-	r = piv_general_io(card, 0x87, priv->alg_id, priv->key_ref, 
+	/* 
+	 * alg_id=06 is a place holder for all RSA keys. 
+ 	 * Derive the real alg_id based on the size of the
+	 * the data, as we are always using raw mode. 
+	 * Non RSA keys needs some work in thia area. 
+	 */
+
+	real_alg_id = priv->alg_id;
+	if (priv->alg_id == 0x06) {
+		switch  (datalen) {
+			case 128: real_alg_id = 0x06; break;
+			case 256: real_alg_id = 0x07; break;
+			case 384: real_alg_id = 0x05; break;
+			default:
+				SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_NO_CARD_SUPPORT);
+		}
+	} 
+
+	r = piv_general_io(card, 0x87, real_alg_id, priv->key_ref, 
 			sbuf, p - sbuf, &rbuf, &rbuflen);  
 
 	if ( r >= 0) {
@@ -1199,7 +1478,7 @@ static int piv_compute_signature(sc_card_t *card,
 					u8 * out, size_t outlen)
 {
 	SC_FUNC_CALLED(card->ctx,4);
-	return piv_validate_general_authentication(card, data, datalen, out, outlen);
+	SC_FUNC_RETURN(card->ctx, 4, piv_validate_general_authentication(card, data, datalen, out, outlen));
 }
 
 static int piv_decipher(sc_card_t *card,
@@ -1208,7 +1487,7 @@ static int piv_decipher(sc_card_t *card,
 {
 	SC_FUNC_CALLED(card->ctx,4);
 
-	return piv_validate_general_authentication(card, data, datalen, out, outlen);
+	SC_FUNC_RETURN(card->ctx, 4, piv_validate_general_authentication(card, data, datalen, out, outlen));
 }
 
 
@@ -1216,7 +1495,7 @@ static int piv_find_obj_by_containerid(sc_card_t *card, const u8 * str)
 {
 	int i;
 
-	SC_FUNC_CALLED(card->ctx,4)
+	SC_FUNC_CALLED(card->ctx,4);
 	sc_debug(card->ctx, "str=0x%02X%02X\n", str[0], str[1]);
 
 	for (i = 0; piv_objects[i].enumtag; i++) {
@@ -1238,13 +1517,14 @@ static int piv_find_obj_by_containerid(sc_card_t *card, const u8 * str)
 static int piv_select_file(sc_card_t *card, const sc_path_t *in_path,
 	sc_file_t **file_out)
 {
- 	struct piv_private_data * priv = (struct piv_private_data *) card->drv_data;
+ 	piv_private_data_t * priv = PIV_DATA(card);
+	piv_cache_item* item = NULL;
 	int i;
 	const u8 *path;
 	int pathlen;
 	sc_file_t *file = NULL;
 	
-	SC_FUNC_CALLED(card->ctx,1)
+	SC_FUNC_CALLED(card->ctx,1);
 
 	path = in_path->value;
 	pathlen = in_path->len;
@@ -1256,7 +1536,7 @@ static int piv_select_file(sc_card_t *card, const sc_path_t *in_path,
 		pathlen -= 2;
 	}
 	if (pathlen != 2)
-			return SC_ERROR_INVALID_ARGUMENTS;
+		SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_INVALID_ARGUMENTS);
 	 
 	i = piv_find_obj_by_containerid(card, path);
 
@@ -1265,8 +1545,15 @@ static int piv_select_file(sc_card_t *card, const sc_path_t *in_path,
 	priv->selected_obj = i;
 	priv->eof = 0;
 
+	/* check out the cache */
+	priv->current_item = NULL;
+	item = get_cache_item(priv, i);
+	if(item) {
+		priv->current_item = item;
+	}
+	
 	/* make it look like the file was found. We don't want to read it now,. */ 
-
+	
 	if (file_out) {
 		file = sc_file_new();
 		if (file == NULL)
@@ -1276,8 +1563,13 @@ static int piv_select_file(sc_card_t *card, const sc_path_t *in_path,
 		/* this could be like the FCI */
 		file->type =  SC_FILE_TYPE_DF;
 		file->shareable = 0;
-		file->ef_structure = 0; 
-		file->size = piv_objects[i].maxlen;
+		file->ef_structure = 0;
+		if(item) {
+			file->size = item->length;
+		} else {
+			file->size = piv_objects[i].maxlen;
+		}
+
 		file->id = (piv_objects[i].containerid[0]<<8) + piv_objects[i].containerid[1];
 
 		*file_out = file;
@@ -1290,12 +1582,16 @@ static int piv_select_file(sc_card_t *card, const sc_path_t *in_path,
 
 static int piv_finish(sc_card_t *card)
 {
- 	struct piv_private_data * priv = (struct piv_private_data *) card->drv_data;
+ 	piv_private_data_t * priv = PIV_DATA(card);
 
 	SC_FUNC_CALLED(card->ctx,1);
 	if (priv) {
 		if (priv->aid_file)
 			sc_file_free(priv->aid_file);
+		if(priv->cache) {
+			free_cache_items(priv);
+			free(priv->cache);
+		}
 		free(priv);
 	}
 	return 0;
@@ -1304,9 +1600,18 @@ static int piv_finish(sc_card_t *card)
 
 static int piv_match_card(sc_card_t *card)
 {
+	int i;
+	sc_file_t aidfile;
 	SC_FUNC_CALLED(card->ctx,1);
-	/* dee need to look at AID */
-	return 0; /* never match */
+	/* Since we send an APDU, the card's logout function may be called...
+	 * however it may be in dirty memory */
+	card->ops->logout = NULL;
+
+	/* Detect by selecting applet */
+	sc_ctx_suppress_errors_on(card->ctx);
+	i = !(piv_find_aid(card, &aidfile));
+	sc_ctx_suppress_errors_off(card->ctx);
+	return i; /* never match */
 }
 
 
@@ -1314,16 +1619,17 @@ static int piv_init(sc_card_t *card)
 {
 	int r;
 	unsigned long flags;
-	struct piv_private_data *priv;
+	piv_private_data_t *priv;
 
 	SC_FUNC_CALLED(card->ctx,1);
-	priv = (struct piv_private_data *) calloc(1, sizeof(struct piv_private_data));
+	priv = (piv_private_data_t *) calloc(1, sizeof(piv_private_data_t));
 
 	if (!priv)
-		return SC_ERROR_OUT_OF_MEMORY;
+		SC_FUNC_RETURN(card->ctx, 1, SC_ERROR_OUT_OF_MEMORY);
 	priv->aid_file = sc_file_new();
 	priv->selected_obj = -1;
-	priv->max_recv_size = card->max_recv_size;
+	priv->max_recv_size = 256;
+	//priv->max_recv_size = card->max_recv_size;
 	priv->max_send_size = card->max_send_size;
 	card->max_recv_size = 0xffff; /* must force pkcs15 read_binary in one call */
 	card->max_send_size = 0xffff;
@@ -1349,6 +1655,9 @@ static int piv_init(sc_card_t *card)
 	_sc_card_add_rsa_alg(card, 3072, flags, 0); /* optional */
 	
 	card->caps |= SC_CARD_CAP_RNG;
+	
+	priv->cache = calloc(PIV_CACHE_SIZE, sizeof(piv_cache_item));
+	priv->cacheLen = PIV_CACHE_SIZE;
 
 	if (r > 0)
 		r = 0;
@@ -1356,6 +1665,29 @@ static int piv_init(sc_card_t *card)
 }
 
 
+static int piv_pin_cmd(sc_card_t *card, struct sc_pin_cmd_data *data, 
+                       int *tries_left)
+{
+	/* Extra validation of (new) PIN during a PIN change request, to
+	 * ensure it's not outside the FIPS 201 4.1.6.1 (numeric only) and
+	 * FIPS 140-2 (6 character minimum) requirements.
+	 */
+	struct sc_card_driver *iso_drv = sc_get_iso7816_driver();
+	if (data->cmd == SC_PIN_CMD_CHANGE) {
+		int i = 0;
+		if (data->pin2.len < 6) {
+			return SC_ERROR_INVALID_PIN_LENGTH;
+		}
+		for(i=0; i < data->pin2.len; ++i) {
+			if (!isdigit(data->pin2.data[i])) {
+				return SC_ERROR_INVALID_DATA;
+			}
+		}
+	}
+	return iso_drv->ops->pin_cmd(card, data, tries_left);
+}
+
+
 static struct sc_card_driver * sc_get_driver(void)
 {
 	struct sc_card_driver *iso_drv = sc_get_iso7816_driver();
@@ -1366,7 +1698,6 @@ static struct sc_card_driver * sc_get_driver(void)
 	piv_ops.finish = piv_finish;
 	
 	piv_ops.select_file =  piv_select_file; /* must use get/put, could emulate? */
-	piv_ops.logout = piv_logout;
 	piv_ops.get_challenge = piv_get_challenge;
 	piv_ops.read_binary = piv_read_binary;
 	piv_ops.write_binary = piv_write_binary;
@@ -1375,6 +1706,7 @@ static struct sc_card_driver * sc_get_driver(void)
 	piv_ops.compute_signature = piv_compute_signature;
 	piv_ops.decipher =  piv_decipher;
 	piv_ops.card_ctl = piv_card_ctl;
+	piv_ops.pin_cmd = piv_pin_cmd;
 
 	return &piv_drv;
 }
