@@ -121,7 +121,25 @@ enum {
 	SC_CARDCTL_INCRYPTO34_PUT_DATA_SECI,
 	SC_CARDCTL_INCRYPTO34_GENERATE_KEY,
 	SC_CARDCTL_INCRYPTO34_CHANGE_KEY_DATA,
-	SC_CARDCTL_INCRYPTO34_ERASE_FILES
+	SC_CARDCTL_INCRYPTO34_ERASE_FILES,
+	
+	/*
+	 * Muscle specific calls
+	 */
+	SC_CARDCTL_MUSCLE_BASE = _CTL_PREFIX('M','S','C'),
+	SC_CARDCTL_MUSCLE_GENERATE_KEY,
+	SC_CARDCTL_MUSCLE_EXTRACT_KEY,
+	SC_CARDCTL_MUSCLE_IMPORT_KEY,
+	SC_CARDCTL_MUSCLE_VERIFIED_PINS,
+
+	/*
+	 * ASEPCOS specific calls
+	 */
+	SC_CARDCTL_ASEPCOS_BASE = _CTL_PREFIX('A','S','E'),
+	SC_CARDCTL_ASEPCOS_CHANGE_KEY,
+	SC_CARDCTL_ASEPCOS_AKN2FILEID,
+	SC_CARDCTL_ASEPCOS_SET_SATTR,
+	SC_CARDCTL_ASEPCOS_ACTIVATE_FILE
 };
 
 enum {
@@ -302,13 +320,19 @@ struct sc_cardctl_oberthur_genkey_info {
 	unsigned long   exponent;
 	unsigned char * pubkey;
 	unsigned int    pubkey_len;
+
+	int     method;     /* SC_AC_XXX */
+	int     key_ref;    /* key reference */		
 };
 
 struct sc_cardctl_oberthur_updatekey_info {
 	enum SC_CARDCTL_OBERTHUR_KEY_TYPE  type;
-	unsigned int    component;
-	const unsigned char *data;
-	unsigned int    len;
+
+	unsigned char   *data;
+	unsigned int    data_len;
+
+	unsigned char   id[256];
+	unsigned int    id_len;
 };
 
 struct sc_cardctl_oberthur_createpin_info {
@@ -347,6 +371,61 @@ struct sc_cardctl_setcos_gen_store_key_info {
 	unsigned char  *primeq;
 };
 
+/*
+ * Muscle stuff
+ */
+typedef struct sc_cardctl_muscle_gen_key_info {
+	int 	keyType;
+	int 	keySize;
+	int 	privateKeyLocation;
+	int 	publicKeyLocation;
+} sc_cardctl_muscle_gen_key_info_t;
+
+
+typedef struct sc_cardctl_muscle_key_info {
+	int 	keyType;
+	int 	keyLocation;
+	int 	keySize;
+	int 	modLength;
+	u8* 	modValue;
+	int 	expLength;
+	u8* 	expValue;
+	int 	pLength;
+	u8* 	pValue;
+	int 	qLength;
+	u8* 	qValue;
+	int 	pqLength;
+	u8* 	pqValue;
+	int 	dp1Length;
+	u8* 	dp1Value;
+	int 	dq1Length;
+	u8* 	dq1Value;
+	int 	gLength;
+	u8* 	gValue;
+	int 	yLength;
+	u8* 	yValue;
+} sc_cardctl_muscle_key_info_t;
+
+typedef struct sc_cardctl_muscle_verified_pins_info {
+	unsigned	verifiedPins;
+} sc_cardctl_muscle_verified_pins_info_t;
+
+/* ASEPCOS ctl specific structures */
+typedef struct sc_cardctl_asepcos_change_key {
+	const u8 *data;
+	size_t datalen;
+} sc_cardctl_asepcos_change_key_t;
+
+typedef struct sc_cardctl_asepcos_akn2fileid {
+	int akn;
+	int fileid;
+} sc_cardctl_asepcos_akn2fileid_t;
+
+typedef struct sc_cardctl_asepcos_activate_file {
+	int	fileid;
+	int	is_ef;
+} sc_cardctl_asepcos_activate_file_t;
+
 #ifdef __cplusplus
 }
 #endif
