@@ -1,7 +1,7 @@
 /*
  * log.h: Logging functions header file
  *
- * Copyright (C) 2001, 2002  Juha Yrjl <juha.yrjola@iki.fi>
+ * Copyright (C) 2001, 2002  Juha Yrjölä <juha.yrjola@iki.fi>
  * Copyright (C) 2003  Antti Tapaninen <aet@cc.hut.fi>
  *
  * This library is free software; you can redistribute it and/or
@@ -55,12 +55,12 @@ void sc_do_log_va(struct sc_context *ctx, int type, const char *file, int line,
 
 void sc_hex_dump(struct sc_context *ctx, const u8 * buf, size_t len, char *out, size_t outlen);
 
-#define SC_FUNC_CALLED(ctx, level) { \
+#define SC_FUNC_CALLED(ctx, level) do { \
 	if (ctx->debug >= level) \
 		 sc_do_log(ctx, SC_LOG_TYPE_DEBUG, __FILE__, __LINE__, __FUNCTION__, "called\n"); \
-}
+} while (0)
 
-#define SC_FUNC_RETURN(ctx, level, r) { \
+#define SC_FUNC_RETURN(ctx, level, r) do { \
 	int _ret = r; \
 	if (_ret < 0 && !ctx->suppress_errors) { \
 		sc_do_log(ctx, SC_LOG_TYPE_ERROR, __FILE__, __LINE__, __FUNCTION__, "returning with: %s\n", sc_strerror(_ret)); \
@@ -68,15 +68,15 @@ void sc_hex_dump(struct sc_context *ctx, const u8 * buf, size_t len, char *out,
 		sc_do_log(ctx, SC_LOG_TYPE_DEBUG, __FILE__, __LINE__, __FUNCTION__, "returning with: %d\n", _ret); \
 	} \
 	return _ret; \
-}
+} while(0)
 
-#define SC_TEST_RET(ctx, r, text) { \
+#define SC_TEST_RET(ctx, r, text) do { \
 	int _ret = (r); \
 	if (_ret < 0) { \
 		sc_do_log(ctx, SC_LOG_TYPE_ERROR, __FILE__, __LINE__, __FUNCTION__, "%s: %s\n", (text), sc_strerror(_ret)); \
 		return _ret; \
 	} \
-}
+} while(0)
 
 #define sc_perror(ctx, errno, str) { \
 	sc_do_log(ctx, SC_LOG_TYPE_ERROR, __FILE__, __LINE__, __FUNCTION__, "%s: %s\n", str, sc_strerror(errno)); \
