@@ -31,6 +31,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include "strlcpy.h"
 
 #ifdef HAVE_ZLIB_H
 #include <zlib.h>
@@ -100,7 +101,7 @@ static int sc_pkcs15emu_add_pin(sc_pkcs15_card_t *p15card,
         if (type == SC_PKCS15_PIN_TYPE_BCD)
                 info.stored_length /= 2;
 
-	strncpy(obj.label, label, SC_PKCS15_MAX_LABEL_SIZE-1);
+	strlcpy(obj.label, label, sizeof(obj.label));
 	obj.flags = obj_flags;
 
         return sc_pkcs15emu_add_pin_obj(p15card, &obj, &info);
@@ -133,7 +134,7 @@ static int sc_pkcs15emu_add_prkey(sc_pkcs15_card_t *p15card,
                 info.path = *path;
 
 	obj.flags = obj_flags;
-	strncpy(obj.label, label, SC_PKCS15_MAX_LABEL_SIZE-1);
+	strlcpy(obj.label, label, sizeof(obj.label));
 	if (auth_id != NULL)
 		obj.auth_id = *auth_id;
 
@@ -242,7 +243,7 @@ static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 		cert_info.path = cpath;
 		cert_info.authority = (i>0);
 
-		strncpy(cert_obj.label, certLabel[i], SC_PKCS15_MAX_LABEL_SIZE - 1);
+		strlcpy(cert_obj.label, certLabel[i], sizeof(cert_obj.label));
 		cert_obj.flags = SC_PKCS15_CO_FLAG_MODIFIABLE;
 
 		sc_pkcs15emu_add_x509_cert(p15card, &cert_obj, &cert_info);
