@@ -26,6 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include "strlcpy.h"
 
 #define MANU_ID		"GemSAFE on GPK16000"
 
@@ -172,7 +173,7 @@ static int sc_pkcs15emu_gemsafe_init(sc_pkcs15_card_t *p15card)
 {
 	const cdata certs[] = {
 		{"User certificate",0, "","1", 0},
-		{NULL, 0, NULL, 0, 0}
+		{NULL, 0, NULL, NULL, 0}
 	};
 
 	const pindata pins[] = {
@@ -185,8 +186,8 @@ static int sc_pkcs15emu_gemsafe_init(sc_pkcs15_card_t *p15card)
 	};
 
 	const prdata prkeys[] = {
-		{ "1", "AUTH key", 1024, USAGE_AUT, "3F0002000009",
-		  0x00, "1", SC_PKCS15_CO_FLAG_PRIVATE},
+		{ "1", "AUTH key", 1024, USAGE_AUT, "I0009",
+		  0x00, "1", 0},
 		{ NULL, NULL, 0, 0, NULL, 0, NULL, 0}
 	};
 
@@ -208,6 +209,13 @@ static int sc_pkcs15emu_gemsafe_init(sc_pkcs15_card_t *p15card)
 
 	SC_FUNC_CALLED(card->ctx, 1);
 
+	/* need to limit to 248 */
+	if (card->max_send_size > 248)
+		card->max_send_size = 248;
+	if (card->max_recv_size > 248)
+		card->max_recv_size = 248;
+
+
 	/* could read this off card if needed */
 
 	p15card->label = strdup("GemSAFE");
@@ -338,7 +346,7 @@ static int sc_pkcs15emu_gemsafe_init(sc_pkcs15_card_t *p15card)
 		sc_pkcs15_format_id(certs[i].id, &cert_info.id);
 		cert_info.authority = certs[i].authority;
 
-		strncpy(cert_obj.label, certs[i].label, SC_PKCS15_MAX_LABEL_SIZE - 1);
+		strlcpy(cert_obj.label, certs[i].label, sizeof(cert_obj.label));
 		cert_obj.flags = certs[i].obj_flags;
 
 		while (idx1 < file->size - 16) { /* actually 13 for all these tests */
@@ -444,7 +452,7 @@ static int sc_pkcs15emu_gemsafe_init(sc_pkcs15_card_t *p15card)
 		pin_info.path.value[3] = dfpath & 0xff;
 		pin_info.tries_left    = -1;
 
-		strncpy(pin_obj.label, pins[i].label, SC_PKCS15_MAX_LABEL_SIZE - 1);
+		strlcpy(pin_obj.label, pins[i].label, sizeof(pin_obj.label));
 		pin_obj.flags = pins[i].obj_flags;
 
 		r = sc_pkcs15emu_add_pin_obj(p15card, &pin_obj, &pin_info);
@@ -472,9 +480,6 @@ static int sc_pkcs15emu_gemsafe_init(sc_pkcs15_card_t *p15card)
 
 		/*DEE need to look for them by reading and checking mudulus vs cert */
 
-		prkey_info.path.value[2] = dfpath >> 8;
-		prkey_info.path.value[3] = dfpath & 0xff;
-
  		/* will use the default path, unless we found a key with */
 		/* the same modulus as the cert(s) we already added */
 		/* This allows us to have a card with a key but no cert */
@@ -483,13 +488,13 @@ static int sc_pkcs15emu_gemsafe_init(sc_pkcs15_card_t *p15card)
 			if (sc_pkcs15_compare_id(&kinfo[j].id, &prkey_info.id))  {
 				sc_debug(card->ctx, "found key in file %d for id %d", 
 						kinfo[j].fileid, prkey_info.id);
-				prkey_info.path.value[4] = kinfo[j].fileid >> 8;
-				prkey_info.path.value[5] = kinfo[j].fileid & 0xff;
+				prkey_info.path.value[0] = kinfo[j].fileid >> 8;
+				prkey_info.path.value[1] = kinfo[j].fileid & 0xff;
 				break;
 			}
 		}
 
-		strncpy(prkey_obj.label, prkeys[i].label, SC_PKCS15_MAX_LABEL_SIZE - 1);
+		strlcpy(prkey_obj.label, prkeys[i].label, sizeof(prkey_obj.label));
 		prkey_obj.flags = prkeys[i].obj_flags;
 		if (prkeys[i].auth_id)
 			sc_pkcs15_format_id(prkeys[i].auth_id, &prkey_obj.auth_id);
