@@ -65,7 +65,7 @@ static struct {
 	{ "actalis",	sc_pkcs15emu_actalis_init_ex	},
 	{ "atrust-acos",sc_pkcs15emu_atrust_acos_init_ex},
 	{ "tccardos",	sc_pkcs15emu_tccardos_init_ex	},
-	{ NULL, 0 }
+	{ NULL, NULL }
 };
 
 static int parse_emu_block(sc_pkcs15_card_t *, scconf_block *);
@@ -77,6 +77,16 @@ static const char *func_name    = "sc_pkcs15_init_func";
 static const char *exfunc_name  = "sc_pkcs15_init_func_ex";
 
 
+int sc_pkcs15_is_emulation_only(sc_card_t *card)
+{
+	switch (card->type) {
+		case SC_CARD_TYPE_MCRD_ESTEID:
+			return 1;
+		default:
+			return 0;
+	}
+}
+
 int
 sc_pkcs15_bind_synthetic(sc_pkcs15_card_t *p15card)
 {
