@@ -1,7 +1,7 @@
 /*
  * reader-ctapi.c: Reader driver for CT-API
  *
- * Copyright (C) 2002  Juha Yrjl <juha.yrjola@iki.fi>
+ * Copyright (C) 2002  Juha Yrjölä <juha.yrjola@iki.fi>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -332,10 +332,8 @@ static int ctapi_transmit(sc_reader_t *reader, sc_slot_info_t *slot,
 	r = sc_apdu_get_octets(reader->ctx, apdu, &sbuf, &ssize, SC_PROTO_RAW);
 	if (r != SC_SUCCESS)
 		goto out;
-	/* log data if DEBUG is defined */
-#ifdef DEBUG
-	sc_apdu_log(reader->ctx, sbuf, ssize, 1);
-#endif
+	if (reader->ctx->debug >= 6)
+		sc_apdu_log(reader->ctx, sbuf, ssize, 1);
 	r = ctapi_internal_transmit(reader, slot, sbuf, ssize,
 					rbuf, &rsize, apdu->control);
 	if (r < 0) {
@@ -343,10 +341,8 @@ static int ctapi_transmit(sc_reader_t *reader, sc_slot_info_t *slot,
 		sc_error(reader->ctx, "unable to transmit");
 		goto out;
 	}
-	/* log data if DEBUG is defined */
-#ifdef DEBUG
-	sc_apdu_log(reader->ctx, rbuf, rsize, 0);
-#endif
+	if (reader->ctx->debug >= 6)
+		sc_apdu_log(reader->ctx, rbuf, rsize, 0);
 	/* set response */
 	r = sc_apdu_set_resp(reader->ctx, apdu, rbuf, rsize);
 out:
