@@ -1,7 +1,7 @@
 /*
  * framework-pkcs15.c: PKCS#15 framework and related objects
  *
- * Copyright (C) 2002  Timo Ters <timo.teras@iki.fi>
+ * Copyright (C) 2002  Timo Teräs <timo.teras@iki.fi>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,8 @@
 #include <opensc/keycache.h>
 #endif
 
+extern int hack_enabled;
+
 #define MAX_CACHE_PIN		32
 struct pkcs15_slot_data {
 	struct sc_pkcs15_object *auth_obj;
@@ -254,6 +256,41 @@ __pkcs15_release_object(struct pkcs15_any_object *obj)
 	return 0;
 }
 
+static int public_key_created(struct pkcs15_fw_data *fw_data,
+			      const unsigned int num_objects,
+			      const u8 *id, 
+			      const size_t size_id,
+			      struct pkcs15_any_object **obj2)
+{
+	int found = 0;
+	int ii=0;
+
+	while(ii<num_objects && !found) {
+		if (!fw_data->objects[ii]->p15_object) {
+			ii++;
+			continue;
+		}
+		if ((fw_data->objects[ii]->p15_object->type != SC_PKCS15_TYPE_PUBKEY) && 
+		    (fw_data->objects[ii]->p15_object->type != SC_PKCS15_TYPE_PUBKEY_RSA) &&
+		    (fw_data->objects[ii]->p15_object->type != SC_PKCS15_TYPE_PUBKEY_DSA)) {
+			ii++;
+			continue;
+		}
+		/* XXX this is somewhat dirty as this assumes that the first 
+		 * member of the is the pkcs15 id */
+		if (memcmp(fw_data->objects[ii]->p15_object->data, id, size_id) == 0) {
+			*obj2 = (struct pkcs15_any_object *) fw_data->objects[ii];
+			found=1;
+		} else
+			ii++;
+	}
+  
+	if (found)
+		return SC_SUCCESS;
+	else 
+		return SC_ERROR_OBJECT_NOT_FOUND;      
+}
+
 static int
 __pkcs15_create_cert_object(struct pkcs15_fw_data *fw_data,
 	struct sc_pkcs15_object *cert, struct pkcs15_any_object **cert_object)
@@ -283,12 +320,15 @@ __pkcs15_create_cert_object(struct pkcs15_fw_data *fw_data,
 	object->cert_data = p15_cert;
 
 	/* Corresponding public key */
-	rv = __pkcs15_create_object(fw_data, (struct pkcs15_any_object **) &obj2,
-					NULL, &pkcs15_pubkey_ops,
-					sizeof(struct pkcs15_pubkey_object));
+	rv = public_key_created(fw_data, fw_data->num_objects, p15_info->id.value, p15_info->id.len, (struct pkcs15_any_object **) &obj2);
+	
+	if (rv != SC_SUCCESS)
+	  rv = __pkcs15_create_object(fw_data, (struct pkcs15_any_object **) &obj2,
+				      NULL, &pkcs15_pubkey_ops,
+				      sizeof(struct pkcs15_pubkey_object));
 	if (rv < 0)
-		return rv;
-
+	  return rv;	
+	
 	if (p15_cert) {
 		obj2->pub_data = &p15_cert->key;
 		obj2->pub_data = (sc_pkcs15_pubkey_t *)calloc(1, sizeof(sc_pkcs15_pubkey_t));
@@ -339,6 +379,11 @@ __pkcs15_create_pubkey_object(struct pkcs15_fw_data *fw_data,
 	if (rv >= 0) {
 		object->pub_info = (struct sc_pkcs15_pubkey_info *) pubkey->data;
 		object->pub_data = p15_key;
+		if (p15_key && object->pub_info->modulus_length == 0 
+				&& p15_key->algorithm == SC_ALGORITHM_RSA) {
+			object->pub_info->modulus_length = 
+				8 * p15_key->u.rsa.modulus.len;
+		}
 	}
 
 	if (pubkey_object != NULL)
@@ -447,8 +492,11 @@ __pkcs15_prkey_bind_related(struct pkcs15_fw_data *fw_data, struct pkcs15_prkey_
 			struct pkcs15_pubkey_object *pubkey;
 			
 			pubkey = (struct pkcs15_pubkey_object *) obj;
-			if (sc_pkcs15_compare_id(&pubkey->pub_info->id, id))
+			if (sc_pkcs15_compare_id(&pubkey->pub_info->id, id)) {
 				pk->prv_pubkey = pubkey;
+				if (pk->prv_info->modulus_length == 0)
+					pk->prv_info->modulus_length = pubkey->pub_info->modulus_length;
+			}
 		}
 	}
 }
@@ -604,8 +652,9 @@ static void pkcs15_init_slot(struct sc_pkcs15_card *card,
 	char tmp[64];
 
 	pkcs15_init_token_info(card, &slot->token_info);
-	slot->token_info.flags |= CKF_USER_PIN_INITIALIZED
-				| CKF_TOKEN_INITIALIZED;
+	slot->token_info.flags |= CKF_TOKEN_INITIALIZED;
+	if (auth != NULL)
+		slot->token_info.flags |= CKF_USER_PIN_INITIALIZED;
 	if (card->card->slot->capabilities & SC_SLOT_CAP_PIN_PAD) {
 		slot->token_info.flags |= CKF_PROTECTED_AUTHENTICATION_PATH;
 		sc_pkcs11_conf.cache_pins = 0;
@@ -626,7 +675,7 @@ static void pkcs15_init_slot(struct sc_pkcs15_card *card,
 		}
 		slot->token_info.flags |= CKF_LOGIN_REQUIRED;
 	} else
-		sprintf(tmp, card->label);
+		snprintf(tmp, sizeof(tmp), "%s", card->label);
 	strcpy_bp(slot->token_info.label, tmp, 32);
 
 	if (pin_info && pin_info->magic == SC_PKCS15_PIN_MAGIC) {
@@ -670,6 +719,7 @@ static CK_RV pkcs15_create_tokens(struct sc_pkcs11_card *p11card)
 	struct sc_pkcs11_slot *slot = NULL;
 	int i, rv, reader = p11card->reader;
 	int auth_count;
+	int found_auth_count = 0;
 	unsigned int j;
 
 	rv = sc_pkcs15_get_objects(fw_data->p15_card,
@@ -712,6 +762,9 @@ static CK_RV pkcs15_create_tokens(struct sc_pkcs11_card *p11card)
 	/* Match up related keys and certificates */
 	pkcs15_bind_related_objects(fw_data);
 
+	if (hack_enabled)
+		auth_count = 1;
+
 	for (i = 0; i < auth_count; i++) {
 		struct sc_pkcs15_pin_info *pin_info = NULL;
 
@@ -721,10 +774,17 @@ static CK_RV pkcs15_create_tokens(struct sc_pkcs11_card *p11card)
 		if ((pin_info->flags & SC_PKCS15_PIN_FLAG_SO_PIN) != 0)
 			continue;
 
-		/* Add all the private keys related to this pin */
+		/* Ignore unblocking pins for hacked module */
+		if (hack_enabled && (pin_info->flags & SC_PKCS15_PIN_FLAG_UNBLOCKING_PIN) != 0)
+			continue;
+
+		found_auth_count++;
+
 		rv = pkcs15_create_slot(p11card, auths[i], &slot);
 		if (rv != CKR_OK)
 			return CKR_OK; /* no more slots available for this card */
+
+		/* Add all objects related to this pin */
 		for (j=0; j < fw_data->num_objects; j++) {
 			struct pkcs15_any_object *obj = fw_data->objects[j];
 
@@ -748,6 +808,8 @@ static CK_RV pkcs15_create_tokens(struct sc_pkcs11_card *p11card)
 		}
 	}
 
+	auth_count = found_auth_count;
+
 	/* Add all public objects to a virtual slot without pin protection.
 	 * If there's only 1 pin and the hide_empty_tokens option is set,
 	 * add the public objects to the slot that corresponds to that pin.
@@ -758,6 +820,9 @@ static CK_RV pkcs15_create_tokens(struct sc_pkcs11_card *p11card)
 	/* Add all the remaining objects */
 	for (j = 0; j < fw_data->num_objects; j++) {
 		struct pkcs15_any_object *obj = fw_data->objects[j];
+		/* We only have one pin and only the things related to it. */
+		if (hack_enabled)
+			break;
 
 		if (!(obj->base.flags & SC_PKCS11_OBJECT_SEEN)) {
 			sc_debug(context, "Object %d was not seen previously\n", j);
@@ -1221,6 +1286,88 @@ static CK_RV pkcs15_create_certificate(struct sc_pkcs11_card *p11card,
 out:	return rv;
 }
 
+static CK_RV pkcs15_create_data(struct sc_pkcs11_card *p11card,
+		struct sc_pkcs11_slot *slot,
+		struct sc_profile *profile,
+		CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount,
+		CK_OBJECT_HANDLE_PTR phObject)
+{
+	struct pkcs15_fw_data *fw_data = (struct pkcs15_fw_data *) p11card->fw_data;
+	struct sc_pkcs15init_dataargs args;
+	struct pkcs15_any_object *data_any_obj;
+	struct sc_pkcs15_object	*data_obj;
+	struct sc_pkcs15_pin_info *pin;
+	CK_BBOOL		bValue;
+	int			rc, rv;
+
+	memset(&args, 0, sizeof(args));
+	args.app_oid.value[0] = -1;
+
+	rv = CKR_OK;
+	while (ulCount--) {
+		CK_ATTRIBUTE_PTR attr = pTemplate++;
+
+		switch (attr->type) {
+		/* Skip attrs we already know or don't care for */
+		case CKA_CLASS:
+			break;
+		case CKA_PRIVATE:
+			rv = attr_extract(attr, &bValue, NULL);
+			if (bValue) {
+				pin = slot_data_pin_info(slot->fw_data);
+				if (pin == NULL) {
+					rv = CKR_TEMPLATE_INCOMPLETE;
+					goto out;
+				}
+				args.auth_id = pin->auth_id;
+			}
+			break;
+		case CKA_LABEL:
+			args.label = (char *) attr->pValue;
+			break;
+		case CKA_ID:
+			args.id.len = sizeof(args.id.value);
+			rv = attr_extract(attr, args.id.value, &args.id.len);
+			if (rv != CKR_OK)
+				goto out;
+			break;
+		case CKA_APPLICATION:
+			args.app_label = (char *) attr->pValue;
+			break;
+		case CKA_OBJECT_ID:
+			rv = attr_extract(attr, args.app_oid.value, NULL);
+			if (rv != CKR_OK)
+				goto out;
+			break;
+		case CKA_VALUE:
+			args.der_encoded.len = attr->ulValueLen;
+			args.der_encoded.value = (u8 *) attr->pValue;
+			break;
+		default:
+			/* ignore unknown attrs, or flag error? */
+			continue;
+		}
+	}
+
+	if (args.der_encoded.len == 0) {
+		rv = CKR_TEMPLATE_INCOMPLETE;
+		goto out;
+	}
+
+	rc = sc_pkcs15init_store_data_object(fw_data->p15_card, profile, &args, &data_obj);
+	if (rc < 0) {
+		rv = sc_to_cryptoki_error(rc, p11card->reader);
+		goto out;
+	}
+	/* Create a new pkcs11 object for it */
+	__pkcs15_create_data_object(fw_data, data_obj, &data_any_obj);
+	pkcs15_add_object(slot, data_any_obj, phObject);
+
+	rv = CKR_OK;
+
+out:	return rv;
+}
+
 static CK_RV pkcs15_create_object(struct sc_pkcs11_card *p11card,
 		struct sc_pkcs11_slot *slot,
 		CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount,
@@ -1261,6 +1408,10 @@ static CK_RV pkcs15_create_object(struct sc_pkcs11_card *p11card,
 		rv = pkcs15_create_certificate(p11card, slot, profile,
 				pTemplate, ulCount, phObject);
 		break;
+	case CKO_DATA:
+		rv = pkcs15_create_data(p11card, slot, profile,
+				pTemplate, ulCount, phObject);
+		break;
 	default:
 		rv = CKR_FUNCTION_NOT_SUPPORTED;
 	}
@@ -1908,6 +2059,15 @@ static CK_RV pkcs15_prkey_sign(struct sc_pkcs11_session *ses, void *obj,
 	case CKM_SHA1_RSA_PKCS:
 		flags = SC_ALGORITHM_RSA_PAD_PKCS1 | SC_ALGORITHM_RSA_HASH_SHA1;
 		break;
+	case CKM_SHA256_RSA_PKCS:
+		flags = SC_ALGORITHM_RSA_PAD_PKCS1 | SC_ALGORITHM_RSA_HASH_SHA256;
+		break;
+	case CKM_SHA384_RSA_PKCS:
+		flags = SC_ALGORITHM_RSA_PAD_PKCS1 | SC_ALGORITHM_RSA_HASH_SHA384;
+		break;
+	case CKM_SHA512_RSA_PKCS:
+		flags = SC_ALGORITHM_RSA_PAD_PKCS1 | SC_ALGORITHM_RSA_HASH_SHA512;
+		break;
 	case CKM_RIPEMD160_RSA_PKCS:
 		flags = SC_ALGORITHM_RSA_PAD_PKCS1 | SC_ALGORITHM_RSA_HASH_RIPEMD160;
 		break;
@@ -2264,16 +2424,15 @@ static int pkcs15_dobj_get_value(struct sc_pkcs11_session *session,
 	if (rv < 0)
 		return sc_to_cryptoki_error(rv, reader);
 		
-	if (slot_data_pin_info(data))   {
+	rv = sc_pkcs15_read_data_object(fw_data->p15_card, dobj->info, out_data);
+
+	/* Do we have to try a re-login and then try to sign again? */
+	if (rv == SC_ERROR_SECURITY_STATUS_NOT_SATISFIED) {
 		rv = revalidate_pin(data, session);
-		if (rv < 0)
-			goto done;
+		if (rv == 0)
+			rv = sc_pkcs15_read_data_object(fw_data->p15_card, dobj->info, out_data);
 	}
-	
-	if ((rv = sc_pkcs15_read_data_object(fw_data->p15_card, dobj->info, out_data)) < 0)
-		goto done;
 
-done:
 	sc_unlock(card);
 	if (rv < 0)
 		return sc_to_cryptoki_error(rv, reader);
@@ -2302,7 +2461,7 @@ static CK_RV pkcs15_dobj_get_attribute(struct sc_pkcs11_session *session,
 	case CKA_PRIVATE:
 		check_attribute_buffer(attr, sizeof(CK_BBOOL));
 		*(CK_BBOOL*)attr->pValue =
-			(dobj->base.p15_object->flags & 0x01) != 0;
+			(dobj->base.p15_object->flags & SC_PKCS15_CO_FLAG_PRIVATE) != 0;
 		break;
 	case CKA_MODIFIABLE:
 		check_attribute_buffer(attr, sizeof(CK_BBOOL));
@@ -2358,12 +2517,55 @@ static CK_RV pkcs15_dobj_get_attribute(struct sc_pkcs11_session *session,
 	return CKR_OK;
 }
 
+static CK_RV pkcs15_dobj_destroy(struct sc_pkcs11_session *session, void *object)
+{
+	struct pkcs15_data_object *obj = (struct pkcs15_data_object*) object;
+	struct sc_pkcs11_card *card = session->slot->card;
+	struct pkcs15_fw_data *fw_data = (struct pkcs15_fw_data *) card->fw_data;
+	struct pkcs15_slot_data *data = slot_data(session->slot->fw_data);
+	struct sc_profile *profile = NULL;
+	int reader = session->slot->card->reader;
+	int rv;
+
+	rv = sc_lock(card->card);
+	if (rv < 0)
+		return sc_to_cryptoki_error(rv, card->reader);
+
+	/* Bind the profile */
+	rv = sc_pkcs15init_bind(card->card, "pkcs15", NULL, &profile);
+	if (rv < 0) {
+		sc_unlock(card->card);
+		return sc_to_cryptoki_error(rv, card->reader);
+	}
+
+	/* Add the PINs the user presented so far to the keycache */
+	add_pins_to_keycache(card, session->slot);
+
+	/* Delete object in smartcard */
+	rv = sc_pkcs15init_delete_object(fw_data->p15_card, profile, obj->base.p15_object);
+
+	/* Do we have to try a re-login and then try to delete again? */
+	if (rv == SC_ERROR_SECURITY_STATUS_NOT_SATISFIED) {
+		rv = revalidate_pin(data, session);
+		if (rv == 0)
+			rv = sc_pkcs15init_delete_object(fw_data->p15_card, profile, obj->base.p15_object);
+	}
+
+	sc_pkcs15init_unbind(profile);
+	sc_unlock(card->card);
+
+	if (rv < 0)
+		return sc_to_cryptoki_error(rv, reader);
+
+	return CKR_OK;
+}
+
 struct sc_pkcs11_object_ops pkcs15_dobj_ops = {
 	pkcs15_dobj_release,
 	pkcs15_dobj_set_attribute,
 	pkcs15_dobj_get_attribute,
 	sc_pkcs11_any_cmp_attribute,
-	NULL,
+	pkcs15_dobj_destroy,
 	NULL,
 	NULL,
 	NULL,
@@ -2571,8 +2773,7 @@ revalidate_pin(struct pkcs15_slot_data *data, struct sc_pkcs11_session *ses)
  * FIXME: We should consult the card's algorithm list to
  * find out what operations it supports
  */
-int
-register_mechanisms(struct sc_pkcs11_card *p11card)
+static int register_mechanisms(struct sc_pkcs11_card *p11card)
 {
 	sc_card_t *card = p11card->card;
 	sc_algorithm_info_t *alg_info;
@@ -2668,8 +2869,7 @@ register_mechanisms(struct sc_pkcs11_card *p11card)
 	return CKR_OK;
 }
 
-int
-lock_card(struct pkcs15_fw_data *fw_data)
+static int lock_card(struct pkcs15_fw_data *fw_data)
 {
 	int	rc;
 
@@ -2681,8 +2881,7 @@ lock_card(struct pkcs15_fw_data *fw_data)
 	return rc;
 }
 
-int
-unlock_card(struct pkcs15_fw_data *fw_data)
+static int unlock_card(struct pkcs15_fw_data *fw_data)
 {
 	while (fw_data->locked) {
 		sc_unlock(fw_data->p15_card->card);
