@@ -1,7 +1,7 @@
 /*
  * misc.c: Miscellaneous PKCS#11 library helper functions
  *
- * Copyright (C) 2002  Timo Ters <timo.teras@iki.fi>
+ * Copyright (C) 2002  Timo Teräs <timo.teras@iki.fi>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -236,6 +236,8 @@ CK_RV attr_extract(CK_ATTRIBUTE_PTR pAttr, void *ptr, size_t *sizep)
 			size = sizeof(CKA_CERTIFICATE_TYPE); break;
 		case CKA_MODULUS_BITS:
 			size = sizeof(CK_ULONG); break;
+		case CKA_OBJECT_ID:
+			size = sizeof(struct sc_object_id); break;
 		default:
 			return CKR_FUNCTION_FAILED;
 		}
@@ -317,8 +319,8 @@ void load_pkcs11_parameters(struct sc_pkcs11_config *conf, sc_context_t *ctx)
 	/* Set defaults */
 	conf->num_slots = SC_PKCS11_DEF_SLOTS_PER_CARD;
 	conf->hide_empty_tokens = 0;
-	conf->lock_login = 1;
-	conf->cache_pins = 0;
+	conf->lock_login = 0;
+	conf->cache_pins = 1;
 	conf->soft_keygen_allowed = 1;
 
 	for (i = 0; ctx->conf_blocks[i] != NULL; i++) {
@@ -335,7 +337,7 @@ void load_pkcs11_parameters(struct sc_pkcs11_config *conf, sc_context_t *ctx)
 
 	conf->num_slots = scconf_get_int(conf_block, "num_slots", conf->num_slots);
 	conf->hide_empty_tokens = scconf_get_bool(conf_block, "hide_empty_tokens", 0);
-	conf->lock_login = scconf_get_bool(conf_block, "lock_login", 1);
-	conf->cache_pins = scconf_get_bool(conf_block, "cache_pins", 0);
+	conf->lock_login = scconf_get_bool(conf_block, "lock_login", 0);
+	conf->cache_pins = scconf_get_bool(conf_block, "cache_pins", 1);
 	conf->soft_keygen_allowed = scconf_get_bool(conf_block, "soft_keygen_allowed", 1);
 }
