@@ -622,7 +622,7 @@ static int cardos_put_key(sc_profile_t *profile, sc_card_t *card,
 		pin_id = 0;
 
 	if (key_info->modulus_length > 1024 && (card->type == SC_CARD_TYPE_CARDOS_M4_2 ||
-	    card->type == SC_CARD_TYPE_CARDOS_M4_3)) {
+	    card->type == SC_CARD_TYPE_CARDOS_M4_3 ||card->type == SC_CARD_TYPE_CARDOS_M4_2B)) {
 		r = cardos_store_key_component(card, algorithm, key_id, pin_id, 0,
 			key->p.data, key->p.len, 0, 0);
 		if (r != SC_SUCCESS)
