@@ -1,4 +1,4 @@
-/* Copyright (C) 2001  Juha Yrjl <juha.yrjola@iki.fi> 
+/* Copyright (C) 2001  Juha Yrjölä <juha.yrjola@iki.fi> 
  * All rights reserved.
  *
  * Common functions for test programs
@@ -10,20 +10,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
+#include "my_getopt.h"
 #include <opensc/opensc.h>
 #include "sc-test.h"
 
 sc_context_t *ctx;
 sc_card_t *card;
 
-const struct option	options[] = {
-	{ "reader",             1, 0,           'r' },
-	{ "driver",		1, 0,           'c' },
-	{ "debug",              0, 0,           'd' },
-	{ 0, 0, 0, 0 }
+static const struct option	options[] = {
+	{ "reader",             1, NULL,           'r' },
+	{ "driver",		1, NULL,           'c' },
+	{ "debug",              0, NULL,           'd' },
+	{ NULL, 0, NULL, 0 }
 };
 
 #if 0
