@@ -1,6 +1,8 @@
 #ifndef _SC_TEST_H
 #define _SC_TEST_H
 
+#include <opensc/pkcs15.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -12,7 +14,7 @@ struct sc_pkcs15_object;
 
 int sc_test_init(int *argc, char *argv[]);
 void sc_test_cleanup();
-void sc_test_print_card(const struct sc_pkcs15_card *);
+void sc_test_print_card(const sc_pkcs15_card_t *);
 void sc_test_print_object(const struct sc_pkcs15_object *);
 
 #ifdef __cplusplus
