@@ -26,10 +26,11 @@
 #include <unistd.h>
 #endif
 #include <stdlib.h>
-#include <getopt.h>
+#include <my_getopt.h>
 #include <string.h>
 #include <opensc/opensc.h>
 #include <opensc/asn1.h>
+#include "util.h"
 
 #include "../libopensc/cards.h"
 #include "../libopensc/esteid.h"
@@ -39,20 +40,18 @@ static int stats = 0;
 static char *exec_program = NULL;
 static int exit_status = EXIT_FAILURE;
 
-static struct option const long_options[] = {
-	{"reader", required_argument, 0, 'r'},
-	{"print", no_argument, 0, 'n'},
-	{"exec", required_argument, 0, 'x'},
-	{"stats", no_argument, 0, 't'},
-	{"help", no_argument, 0, 'h'},
-	{"version", no_argument, 0, 'V'},
+static const struct option options[] = {
+	{"reader", required_argument, NULL, 'r'},
+	{"print", no_argument, NULL, 'n'},
+	{"exec", required_argument, NULL, 'x'},
+	{"stats", no_argument, NULL, 't'},
+	{"help", no_argument, NULL, 'h'},
+	{"version", no_argument, NULL, 'V'},
 	{NULL, 0, NULL, 0}
 };
 
 /* Probably not used, but needed to build on Windows */
-const char *app_name = "eidenv";
-const struct option options[] = {{NULL, 0, NULL, 0}};
-const char *option_help[] = {NULL};
+static const char *app_name = "eidenv";
 
 static struct {
 	const char *name;
@@ -103,7 +102,7 @@ static void decode_options(int argc, char **argv)
 {
 	int c;
 
-	while ((c = getopt_long(argc, argv,"ptr:x:hV", long_options, (int *) 0)) != EOF) {
+	while ((c = getopt_long(argc, argv,"ptr:x:hV", options, (int *) 0)) != EOF) {
 
 		switch (c) {
 		case 'r':
