@@ -1,7 +1,7 @@
 /*
  * pkcs15-tool.c: Tool for poking with PKCS #15 smart cards
  *
- * Copyright (C) 2001  Juha Yrjl <juha.yrjola@iki.fi>
+ * Copyright (C) 2001  Juha Yrjölä <juha.yrjola@iki.fi>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,18 +38,18 @@ typedef unsigned __int32 uint32_t;
 #include <opensc/pkcs15.h>
 #include "util.h"
 
-const char *app_name = "pkcs15-tool";
+static const char *app_name = "pkcs15-tool";
 
-int opt_reader = -1, opt_wait = 0;
-int opt_no_cache = 0;
-char * opt_auth_id;
-char * opt_cert = NULL;
-char * opt_data = NULL;
-char * opt_pubkey = NULL;
-char * opt_outfile = NULL;
-u8 * opt_newpin = NULL;
-u8 * opt_pin = NULL;
-u8 * opt_puk = NULL;
+static int opt_reader = -1, opt_wait = 0;
+static int opt_no_cache = 0;
+static char * opt_auth_id;
+static char * opt_cert = NULL;
+static char * opt_data = NULL;
+static char * opt_pubkey = NULL;
+static char * opt_outfile = NULL;
+static u8 * opt_newpin = NULL;
+static u8 * opt_pin = NULL;
+static u8 * opt_puk = NULL;
 
 static int	verbose = 0;
 
@@ -72,38 +72,37 @@ enum {
 #define NELEMENTS(x)	(sizeof(x)/sizeof((x)[0]))
 
 static int	authenticate(sc_pkcs15_object_t *obj);
-static int	pem_encode(sc_context_t *, int,
-			sc_pkcs15_der_t *, sc_pkcs15_der_t *);
+static int	pem_encode(int, sc_pkcs15_der_t *, sc_pkcs15_der_t *);
 
-const struct option options[] = {
-	{ "learn-card",		no_argument, 0, 	'L' },
-	{ "read-certificate",	required_argument, 0, 	'r' },
-	{ "list-certificates",	no_argument, 0,		'c' },
-	{ "read-data-object",	required_argument, 0, 	'R' },
-	{ "list-data-objects",	no_argument, 0,		'C' },
-	{ "list-pins",		no_argument, 0,		OPT_LIST_PINS },
-	{ "dump",		no_argument, 0,		'D' },
-	{ "unblock-pin",	no_argument, 0,		'u' },
-	{ "change-pin",		no_argument, 0,		OPT_CHANGE_PIN },
-	{ "list-keys",          no_argument, 0,         'k' },
-	{ "list-public-keys",	no_argument, 0,		OPT_LIST_PUB },
-	{ "read-public-key",	required_argument, 0,	OPT_READ_PUB },
+static const struct option options[] = {
+	{ "learn-card",		no_argument, NULL, 	'L' },
+	{ "read-certificate",	required_argument, NULL, 	'r' },
+	{ "list-certificates",	no_argument, NULL,		'c' },
+	{ "read-data-object",	required_argument, NULL, 	'R' },
+	{ "list-data-objects",	no_argument, NULL,		'C' },
+	{ "list-pins",		no_argument, NULL,		OPT_LIST_PINS },
+	{ "dump",		no_argument, NULL,		'D' },
+	{ "unblock-pin",	no_argument, NULL,		'u' },
+	{ "change-pin",		no_argument, NULL,		OPT_CHANGE_PIN },
+	{ "list-keys",          no_argument, NULL,         'k' },
+	{ "list-public-keys",	no_argument, NULL,		OPT_LIST_PUB },
+	{ "read-public-key",	required_argument, NULL,	OPT_READ_PUB },
 #if defined(HAVE_OPENSSL) && (defined(_WIN32) || defined(HAVE_INTTYPES_H))
-	{ "read-ssh-key",	required_argument, 0,	OPT_READ_SSH },
+	{ "read-ssh-key",	required_argument, NULL,	OPT_READ_SSH },
 #endif
-	{ "reader",		required_argument, 0,	OPT_READER },
-	{ "pin",                required_argument, 0,   OPT_PIN },
-	{ "new-pin",		required_argument, 0,	OPT_NEWPIN },
-	{ "puk",		required_argument, 0,	OPT_PUK },
-	{ "output",		required_argument, 0,	'o' },
-	{ "no-cache",		no_argument, 0,		OPT_NO_CACHE },
-	{ "auth-id",		required_argument, 0,	'a' },
-	{ "wait",		no_argument, 0,		'w' },
-	{ "verbose",		no_argument, 0,		'v' },
-	{ 0, 0, 0, 0 }
+	{ "reader",		required_argument, NULL,	OPT_READER },
+	{ "pin",                required_argument, NULL,   OPT_PIN },
+	{ "new-pin",		required_argument, NULL,	OPT_NEWPIN },
+	{ "puk",		required_argument, NULL,	OPT_PUK },
+	{ "output",		required_argument, NULL,	'o' },
+	{ "no-cache",		no_argument, NULL,		OPT_NO_CACHE },
+	{ "auth-id",		required_argument, NULL,	'a' },
+	{ "wait",		no_argument, NULL,		'w' },
+	{ "verbose",		no_argument, NULL,		'v' },
+	{ NULL, 0, NULL, 0 }
 };
 
-const char *option_help[] = {
+static const char *option_help[] = {
 	"Stores card info to cache",
 	"Reads certificate with ID <arg>",
 	"Lists certificates",
@@ -128,9 +127,9 @@ const char *option_help[] = {
 	"Verbose operation. Use several times to enable debug output.",
 };
 
-sc_context_t *ctx = NULL;
-sc_card_t *card = NULL;
-struct sc_pkcs15_card *p15card = NULL;
+static sc_context_t *ctx = NULL;
+static sc_card_t *card = NULL;
+static struct sc_pkcs15_card *p15card = NULL;
 
 static void print_cert_info(const struct sc_pkcs15_object *obj)
 {
@@ -324,16 +323,22 @@ static int read_data_object(void)
 			
 		if (verbose)
 			printf("Reading data object with label '%s'\n", opt_data);
-		r = sc_pkcs15_read_data_object(p15card, cinfo, &data_object);
-		if (r) {
-			fprintf(stderr, "Data object read failed: %s\n", sc_strerror(r));
-			if (r == SC_ERROR_FILE_NOT_FOUND)
-				continue; /* DEE emulation may say there is a file */
+		r = authenticate(objs[i]);
+		if (r >= 0) {
+			r = sc_pkcs15_read_data_object(p15card, cinfo, &data_object);
+			if (r) {
+				fprintf(stderr, "Data object read failed: %s\n", sc_strerror(r));
+				if (r == SC_ERROR_FILE_NOT_FOUND)
+					continue; /* DEE emulation may say there is a file */
+				return 1;
+			}
+			r = print_data_object("Data Object", data_object->data, data_object->data_len);
+			sc_pkcs15_free_data_object(data_object);
+			return r;
+		} else {
+			fprintf(stderr, "Authentication error: %s\n", sc_strerror(r));
 			return 1;
 		}
-		r = print_data_object("Data Object", data_object->data, data_object->data_len);
-		sc_pkcs15_free_data_object(data_object);
-		return r;
 	}
 	fprintf(stderr, "Data object with label '%s' not found.\n", opt_data);
 	return 2;
@@ -353,7 +358,6 @@ static int list_data_objects(void)
 	for (i = 0; i < count; i++) {
 		int idx;
 		struct sc_pkcs15_data_info *cinfo = (struct sc_pkcs15_data_info *) objs[i]->data;
-		struct sc_pkcs15_data *data_object;
 
 		printf("Reading data object <%i>\n", i);
 		printf("applicationName: %s\n", cinfo->app_label);
@@ -369,16 +373,20 @@ static int list_data_objects(void)
 			printf("\n");
 		} else
 			printf("NONE\n");
-		printf("Path :           %s\n", sc_print_path(&cinfo->path));
-		r = sc_pkcs15_read_data_object(p15card, cinfo, &data_object);
-		if (r) {
-			fprintf(stderr, "Data object read failed: %s\n", sc_strerror(r));
-			if (r == SC_ERROR_FILE_NOT_FOUND)
-				 continue; /* DEE emulation may say there is a file */
-			return 1;
+		printf("Path:            %s\n", sc_print_path(&cinfo->path));
+		printf("Auth ID:         %s\n", sc_pkcs15_print_id(&objs[i]->auth_id));
+		if (objs[i]->auth_id.len == 0) {
+			struct sc_pkcs15_data *data_object;
+			r = sc_pkcs15_read_data_object(p15card, cinfo, &data_object);
+			if (r) {
+				fprintf(stderr, "Data object read failed: %s\n", sc_strerror(r));
+				if (r == SC_ERROR_FILE_NOT_FOUND)
+					 continue; /* DEE emulation may say there is a file */
+				return 1;
+			}
+			r = list_data_object("Data Object", data_object->data, data_object->data_len);
+			sc_pkcs15_free_data_object(data_object);
 		}
-		r = list_data_object("Data Object", data_object->data, data_object->data_len);
-		sc_pkcs15_free_data_object(data_object);
 	}
 	return 0;
 }
@@ -413,7 +421,7 @@ static void print_prkey_info(const struct sc_pkcs15_object *obj)
 			printf(", %s", access_flags[i]);   
 		}
 	printf("\n");
-	printf("\tModLength   : %lu\n", prkey->modulus_length);
+	printf("\tModLength   : %lu\n", (unsigned long)prkey->modulus_length);
 	printf("\tKey ref     : %d\n", prkey->key_reference);
 	printf("\tNative      : %s\n", prkey->native ? "yes" : "no");
 	printf("\tPath        : %s\n", sc_print_path(&prkey->path));
@@ -471,7 +479,7 @@ static void print_pubkey_info(const struct sc_pkcs15_object *obj)
 			printf(", %s", access_flags[i]);   
 		}
 	printf("\n");
-	printf("\tModLength   : %lu\n", pubkey->modulus_length);
+	printf("\tModLength   : %lu\n", (unsigned long)pubkey->modulus_length);
 	printf("\tKey ref     : %d\n", pubkey->key_reference);
 	printf("\tNative      : %s\n", pubkey->native ? "yes" : "no");
 	printf("\tPath        : %s\n", sc_print_path(&pubkey->path));
@@ -544,7 +552,7 @@ static int read_public_key(void)
 		return 1;
 	}
 
-	r = pem_encode(ctx, pubkey->algorithm, &pubkey->data, &pem_key);
+	r = pem_encode(pubkey->algorithm, &pubkey->data, &pem_key);
 	if (r < 0) {
 		fprintf(stderr, "Error encoding PEM key: %s\n",
 				sc_strerror(r));
@@ -839,8 +847,7 @@ static u8 * get_pin(const char *prompt, sc_pkcs15_object_t *pin_obj)
 	}
 }
 
-int
-authenticate(sc_pkcs15_object_t *obj)
+static int authenticate(sc_pkcs15_object_t *obj)
 {
 	sc_pkcs15_pin_info_t	*pin_info;
 	sc_pkcs15_object_t	*pin_obj;
@@ -854,7 +861,10 @@ authenticate(sc_pkcs15_object_t *obj)
 		return r;
 
 	pin_info = (sc_pkcs15_pin_info_t *) pin_obj->data;
-	pin = get_pin("Please enter PIN", pin_obj);
+	if (opt_pin != NULL)
+		pin = opt_pin;
+	else
+		pin = get_pin("Please enter PIN", pin_obj);
 
 	return sc_pkcs15_verify_pin(p15card, pin_info,
 			pin, pin? strlen((char *) pin) : 0);
@@ -885,10 +895,14 @@ static void print_pin_info(const struct sc_pkcs15_object *obj)
 		}
 	printf("\n");
 	printf("\tLength    : min_len:%lu, max_len:%lu, stored_len:%lu\n",
-				pin->min_length, pin->max_length, pin->stored_length);
+		(unsigned long)pin->min_length, (unsigned long)pin->max_length,
+		(unsigned long)pin->stored_length);
 	printf("\tPad char  : 0x%02X\n", pin->pad_char);
 	printf("\tReference : %d\n", pin->reference);
-	printf("\tType      : %s\n", pin_types[pin->type]);
+	if (pin->type >= 0 && pin->type < sizeof(pin_types)/sizeof(pin_types[0]))
+		printf("\tType      : %s\n", pin_types[pin->type]);
+	else
+		printf("\tType      : [encoding %d]\n", pin->type);
 	printf("\tPath      : %s\n", sc_print_path(&pin->path));
 	if (pin->tries_left >= 0)
 		printf("\tTries left: %d\n", pin->tries_left);
@@ -913,7 +927,7 @@ static int list_pins(void)
 	return 0;
 }
 
-static int dump()
+static int dump(void)
 {
 
 	const char *flags[] = {
@@ -1187,7 +1201,7 @@ int main(int argc, char * const argv[])
 		if (c == -1)
 			break;
 		if (c == '?')
-			print_usage_and_die();
+			print_usage_and_die(app_name, options, option_help);
 		switch (c) {
 		case 'r':
 			opt_cert = optarg;
@@ -1277,7 +1291,7 @@ int main(int argc, char * const argv[])
 		}
 	}
 	if (action_count == 0)
-		print_usage_and_die();
+		print_usage_and_die(app_name, options, option_help);
 
 	memset(&ctx_param, 0, sizeof(ctx_param));
 	ctx_param.ver      = 0;
@@ -1400,9 +1414,7 @@ static const struct sc_asn1_entry	c_asn1_pem_key[] = {
 	{ NULL, 0, 0, 0, NULL, NULL }
 };
 
-static int
-pem_encode(sc_context_t *ctx,
-		int alg_id, sc_pkcs15_der_t *key, sc_pkcs15_der_t *out)
+static int pem_encode(int alg_id, sc_pkcs15_der_t *key, sc_pkcs15_der_t *out)
 {
 	struct sc_asn1_entry	asn1_pem_key[2],
 				asn1_pem_key_items[3];
