@@ -16,23 +16,18 @@
 #include <unistd.h>
 #endif
 #include <sys/stat.h>
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
+#include <my_getopt.h>
 #include <opensc/opensc.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-extern const struct option options[];
-extern const char *option_help[];
-extern const char *app_name;
-
 void print_binary(FILE *f, const u8 *buf, int count);
 void hex_dump(FILE *f, const u8 *in, int len, const char *sep);
 void hex_dump_asc(FILE *f, const u8 *in, size_t count, int addr);
-void print_usage_and_die(void);
+void print_usage_and_die(const char *app_name, const struct option options[],
+	const char *option_help[]);
 const char * acl_to_str(const struct sc_acl_entry *e);
 void warn(const char *fmt, ...);
 void error(const char *fmt, ...);
